@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2021 the xine project
+ * Copyright (C) 2000-2023 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -181,9 +181,11 @@ int gui_xine_get_pos_length (gGui_t *gui, xine_stream_t *stream, int *ppos, int
       if (ret == 0)
         break;
       if (stream == gui->stream) {
+        pthread_mutex_lock (&gui->seek_mutex);
         gui->stream_length.pos = lpos;
         gui->stream_length.time = ltime;
         gui->stream_length.length = llength;
+        pthread_mutex_unlock (&gui->seek_mutex);
       }
       if (ppos)
         *ppos = lpos;
@@ -237,6 +239,7 @@ void gui_display_logo (gGui_t *gui) {
   pthread_mutex_unlock(&gui->logo_mutex);
 }
 
+/* return 0 (failure), 1 (ok), 2 (stream started from logo). */
 static int _gui_xine_play (gGui_t *gui, xine_stream_t *stream, int start_pos, int start_ms, int update_mmk) {
   int      ret;
   int      has_video;
@@ -264,9 +267,10 @@ static int _gui_xine_play (gGui_t *gui, xine_stream_t *stream, int start_pos, in
 
   }
 
-  if ((ret = xine_play (stream, start_pos, start_ms)) == 0)
+  ret = xine_play (stream, start_pos, start_ms) ? 1 : 0;
+  if (!ret) {
     gui_handle_xine_error (gui, stream, NULL);
-  else {
+  } else {
     char *ident;
 
     pthread_mutex_lock(&gui->logo_mutex);
@@ -279,14 +283,9 @@ static int _gui_xine_play (gGui_t *gui, xine_stream_t *stream, int start_pos, in
       if (stream_infos_is_visible (gui->streaminfo))
 	stream_infos_update_infos (gui->streaminfo);
 
-      if(update_mmk && ((ident = stream_infos_get_ident_from_stream(stream)) != NULL)) {
-        pthread_mutex_lock (&gui->mmk_mutex);
-	free(gui->mmk.ident);
-	free(gui->playlist.mmk[gui->playlist.cur]->ident);
-
-	gui->mmk.ident = strdup(ident);
-	gui->playlist.mmk[gui->playlist.cur]->ident = strdup(ident);
-        pthread_mutex_unlock (&gui->mmk_mutex);
+      if (update_mmk && ((ident = stream_infos_get_ident_from_stream (stream)) != NULL)) {
+        gui_playlist_set_str_val (gui, ident, MMK_VAL_IDENT, GUI_MMK_CURRENT);
+        gui_current_set_index (gui, GUI_MMK_CURRENT);
 
         video_window_set_mrl (gui->vwin, ident);
         playlist_mrlident_toggle (gui);
@@ -335,55 +334,15 @@ static int _gui_xine_play (gGui_t *gui, xine_stream_t *stream, int start_pos, in
 	  gui->visual_anim.running = 2;
       }
 
-      xine_usec_sleep(100);
-      if(!already_playing)
-        osd_update_status (gui);
+      if (!already_playing)
+        ret = 2;
     }
   }
 
   return ret;
 }
 
-static void free_mmk(gGui_t *gui) {
-  free(gui->mmk.mrl);
-  free(gui->mmk.ident);
-  free(gui->mmk.sub);
-  if(mediamark_have_alternates(&(gui->mmk)))
-    mediamark_free_alternates(&(gui->mmk));
-}
-
-static void set_mmk(gGui_t *gui, mediamark_t *mmk) {
-
-  pthread_mutex_lock (&gui->mmk_mutex);
-  free_mmk(gui);
-
-  if(mmk) {
-    gui->mmk.mrl           = strdup(mmk->mrl);
-    gui->mmk.ident         = strdup(((mmk->ident) ? mmk->ident : mmk->mrl));
-    gui->mmk.sub           = mmk->sub ? strdup(mmk->sub): NULL;
-    gui->mmk.start         = mmk->start;
-    gui->mmk.end           = mmk->end;
-    gui->mmk.av_offset     = mmk->av_offset;
-    gui->mmk.spu_offset    = mmk->spu_offset;
-    gui->mmk.got_alternate = mmk->got_alternate;
-    mediamark_duplicate_alternates(mmk, &(gui->mmk));
-  } else {
-    /* TRANSLATORS: only ASCII characters (skin) */
-    gui->mmk.mrl           = strdup(pgettext("skin", "There is no MRL."));
-    gui->mmk.ident         = strdup ("xine-ui version " VERSION);
-    gui->mmk.sub           = NULL;
-    gui->mmk.start         = 0;
-    gui->mmk.end           = -1;
-    gui->mmk.av_offset     = 0;
-    gui->mmk.spu_offset    = 0;
-    gui->mmk.got_alternate = 0;
-    gui->mmk.alternates    = NULL;
-    gui->mmk.cur_alt       = NULL;
-  }
-  pthread_mutex_unlock (&gui->mmk_mutex);
-}
-
-static void mmk_set_update(gGui_t *gui) {
+void gui_pl_updated (gGui_t *gui) {
 
   video_window_set_mrl (gui->vwin, gui->mmk.ident);
   event_sender_update_menu_buttons (gui);
@@ -397,11 +356,13 @@ static void _start_anyway_done (void *data, int state) {
   gGui_t *gui = data;
   gui->play_data.running = 0;
 
-  if (state == 2)
-    _gui_xine_play (gui, gui->play_data.stream, gui->play_data.start_pos,
-      gui->play_data.start_time_in_secs * 1000, gui->play_data.update_mmk);
-  else
+  if (state == 2) {
+    if (_gui_xine_play (gui, gui->play_data.stream, gui->play_data.start_pos,
+      gui->play_data.start_time_in_secs * 1000, gui->play_data.update_mmk) == 2)
+      osd_update_status (gui);
+  } else {
     gui_playlist_start_next (gui);
+  }
 }
 
 int gui_xine_play (gGui_t *gui, xine_stream_t *stream, int start_pos, int start_time_in_secs, int update_mmk) {
@@ -497,7 +458,12 @@ int gui_xine_play (gGui_t *gui, xine_stream_t *stream, int start_pos, int start_
     free(buffer); free(v_info); free(a_info);
   }
 
-  return _gui_xine_play (gui, stream, start_pos, start_time_in_secs * 1000, update_mmk);
+  {
+    int ret = _gui_xine_play (gui, stream, start_pos, start_time_in_secs * 1000, update_mmk);
+    if (ret == 2)
+      osd_update_status (gui);
+    return ret;
+  }
 }
 
 int gui_xine_open_and_play (gGui_t *gui, char *_mrl, char *_sub, int start_pos,
@@ -539,29 +505,14 @@ int gui_xine_open_and_play (gGui_t *gui, char *_mrl, char *_sub, int start_pos,
 	snprintf(fullfilename, sizeof(fullfilename), "%s/%s", xine_get_homedir(), filename);
 
 	if((fd = fopen(fullfilename, "w+b")) != NULL) {
-	  char  *sub = NULL;
-	  int    start, end;
-
-	  char *newmrl = strdup(fullfilename);
-	  char *ident = strdup(gui->playlist.mmk[gui->playlist.cur]->ident);
-	  if(gui->playlist.mmk[gui->playlist.cur]->sub)
-	    sub = strdup(gui->playlist.mmk[gui->playlist.cur]->sub);
-	  start = gui->playlist.mmk[gui->playlist.cur]->start;
-	  end = gui->playlist.mmk[gui->playlist.cur]->end;
-
 	  fwrite(download.buf, download.size, 1, fd);
 	  fflush(fd);
 	  fclose(fd);
 
-	  mediamark_replace_entry(&gui->playlist.mmk[gui->playlist.cur],
-				  newmrl, ident, sub, start, end, 0, 0);
-          gui_set_current_mmk (gui, mediamark_get_current_mmk (gui));
+          mediamark_set_str_val (&gui->playlist.mmk[gui->playlist.cur], fullfilename, MMK_VAL_MRL);
+          gui_current_set_index (gui, GUI_MMK_CURRENT);
 	  mrl = gui->mmk.mrl;
-
-	  free(newmrl);
-	  free(ident);
-	  free(sub);
-	}
+        }
       }
     }
 
@@ -570,9 +521,9 @@ int gui_xine_open_and_play (gGui_t *gui, char *_mrl, char *_sub, int start_pos,
 
   }
 
-  if(mrl_look_like_playlist(mrl)) {
-    if (mediamark_concat_mediamarks (gui, mrl)) {
-      gui_set_current_mmk (gui, mediamark_get_current_mmk (gui));
+  if (mrl_look_like_playlist (gui, mrl)) {
+    if (gui_playlist_add_item (gui, mrl, 1, GUI_ITEM_TYPE_AUTO, 0)) {
+      gui_current_set_index (gui, GUI_MMK_CURRENT);
       mrl        = gui->mmk.mrl;
       start_pos  = 0;
       start_time = gui->mmk.start;
@@ -706,6 +657,7 @@ void gui_exit_2 (gGui_t *gui) {
   panel_deinit (gui->panel);
   destroy_mrl_browser (gui->mrlb);
   control_deinit (gui->vctrl);
+  acontrol_deinit (gui->actrl);
 
   gui_deinit (gui);
 
@@ -787,8 +739,8 @@ void gui_exit_2 (gGui_t *gui) {
 
   xitk_skin_unload_config(gui->skin_config);
 
-  mediamark_free_mediamarks (gui);
-  free_mmk(gui);
+  gui_playlist_free (gui);
+  gui_current_free (gui);
 
   if (gui->icon)
     xitk_image_free_image (&gui->icon);
@@ -855,7 +807,9 @@ void gui_stop (xitk_widget_t *w, void *data) {
   if(!(gui->playlist.control & PLAYLIST_CONTROL_STOP_PERSIST))
     gui->playlist.control &= ~PLAYLIST_CONTROL_STOP;
 
+  pthread_mutex_lock (&gui->seek_mutex);
   gui->stream_length.pos = gui->stream_length.time = gui->stream_length.length = 0;
+  pthread_mutex_unlock (&gui->seek_mutex);
 
   mediamark_reset_played_state (gui);
   if(gui->visual_anim.running) {
@@ -873,13 +827,8 @@ void gui_stop (xitk_widget_t *w, void *data) {
   panel_update_runtime_display (gui->panel);
 
   if (is_playback_widgets_enabled (gui->panel)) {
-    if(!gui->playlist.num) {
-      gui_set_current_mmk (gui, NULL);
+    if (gui_current_set_index (gui, GUI_MMK_CURRENT) == GUI_MMK_NONE)
       enable_playback_controls (gui->panel, 0);
-    }
-    else if (gui->playlist.num && (strcmp ((mediamark_get_current_mrl (gui)), gui->mmk.mrl))) {
-      gui_set_current_mmk (gui, mediamark_get_current_mmk (gui));
-    }
   }
   gui_display_logo (gui);
 }
@@ -895,7 +844,9 @@ void gui_close (xitk_widget_t *w, void *data) {
   if(!(gui->playlist.control & PLAYLIST_CONTROL_STOP_PERSIST))
     gui->playlist.control &= ~PLAYLIST_CONTROL_STOP;
 
+  pthread_mutex_lock (&gui->seek_mutex);
   gui->stream_length.pos = gui->stream_length.time = gui->stream_length.length = 0;
+  pthread_mutex_unlock (&gui->seek_mutex);
 
   mediamark_reset_played_state (gui);
   if(gui->visual_anim.running) {
@@ -913,13 +864,8 @@ void gui_close (xitk_widget_t *w, void *data) {
   panel_update_runtime_display (gui->panel);
 
   if (is_playback_widgets_enabled (gui->panel)) {
-    if(!gui->playlist.num) {
-      gui_set_current_mmk (gui, NULL);
+    if (gui_current_set_index (gui, GUI_MMK_CURRENT) == GUI_MMK_NONE)
       enable_playback_controls (gui->panel, 0);
-    }
-    else if (gui->playlist.num && (strcmp ((mediamark_get_current_mrl (gui)), gui->mmk.mrl))) {
-      gui_set_current_mmk (gui, mediamark_get_current_mmk (gui));
-    }
   }
   gui->ignore_next = 1;
   xine_close (gui->stream);
@@ -951,7 +897,6 @@ void gui_pause (xitk_widget_t *w, void *data, int state) {
 
 void gui_eject (xitk_widget_t *w, void *data) {
   gGui_t *gui = data;
-  int i;
 
   (void)w;
   if(xine_get_status(gui->stream) == XINE_STATUS_PLAY)
@@ -960,10 +905,8 @@ void gui_eject (xitk_widget_t *w, void *data) {
   if(xine_eject(gui->stream)) {
 
     if(gui->playlist.num) {
-      mediamark_t **mmk = NULL;
       char         *mrl;
       size_t        tok_len = 0;
-      int           new_num = 0;
 
       const char *const cur_mrl = mediamark_get_current_mrl (gui);
       /*
@@ -972,49 +915,28 @@ void gui_eject (xitk_widget_t *w, void *data) {
       if ( (mrl = strstr((cur_mrl), ":/")) )
 	tok_len = (mrl - cur_mrl) + 2;
 
-      if(tok_len != 0) {
-	/*
-	 * Store all of not maching entries
-	 */
-	for(i = 0; i < gui->playlist.num; i++) {
-	  if(strncasecmp(gui->playlist.mmk[i]->mrl, cur_mrl, tok_len)) {
-
-	    mmk = (mediamark_t **) realloc(mmk, sizeof(mediamark_t *) * (new_num + 2));
-
-	    (void) mediamark_store_mmk(&mmk[new_num],
-				       gui->playlist.mmk[i]->mrl,
-				       gui->playlist.mmk[i]->ident,
-				       gui->playlist.mmk[i]->sub,
-				       gui->playlist.mmk[i]->start,
-				       gui->playlist.mmk[i]->end,
-				       gui->playlist.mmk[i]->av_offset,
-				       gui->playlist.mmk[i]->spu_offset);
-	    new_num++;
-	  }
-	}
-
-	/*
-	 * flip playlists.
-	 */
-        mediamark_free_mediamarks (gui);
-	if(mmk)
-	  gui->playlist.mmk = mmk;
-
-	if(!(gui->playlist.num = new_num))
-	  gui->playlist.cur = -1;
-	else if(new_num)
-	  gui->playlist.cur = 0;
+      if (tok_len != 0) {
+        mediamark_t **a, **b, **e;
+        gui_playlist_lock (gui);
+        /* remove maching entries */
+        for (a = b = gui->playlist.mmk, e = a + gui->playlist.num; a < e; a++) {
+          if (!strncasecmp ((*a)->mrl, cur_mrl, tok_len))
+            mediamark_free (a);
+          else
+            *b++ = *a;
+        }
+        gui->playlist.num = b - gui->playlist.mmk;
+        gui->playlist.cur = gui->playlist.num ? 0 : -1;
+        for (; b < e; b++)
+          *b = NULL;
+        gui_playlist_unlock (gui);
       }
       else {
       __remove_current_mrl:
 	/*
 	 * Remove only the current MRL
 	 */
-        pthread_mutex_lock(&gui->mmk_mutex);
-        mediamark_delete_entry (gui, gui->playlist.cur);
-	if(gui->playlist.cur)
-	  gui->playlist.cur--;
-        pthread_mutex_unlock(&gui->mmk_mutex);
+        gui_playlist_remove (gui, GUI_MMK_CURRENT);
       }
 
       if (is_playback_widgets_enabled (gui->panel) && (!gui->playlist.num))
@@ -1022,7 +944,7 @@ void gui_eject (xitk_widget_t *w, void *data) {
 
     }
 
-    gui_set_current_mmk (gui, mediamark_get_current_mmk (gui));
+    gui_current_set_index (gui, GUI_MMK_CURRENT);
     playlist_update_playlist (gui);
   }
   else {
@@ -1253,15 +1175,26 @@ static void *gui_seek_thread (void *data) {
 
     gui->ignore_next = 1;
 
+    gui_playlist_lock (gui);
     if ( gui->playlist.num &&
         (gui->playlist.cur >= 0) &&
          gui->playlist.mmk &&
          gui->playlist.mmk[gui->playlist.cur] &&
         !strcmp (gui->playlist.mmk[gui->playlist.cur]->mrl, gui->mmk.mrl))
       update_mmk = 1;
+    gui_playlist_unlock (gui);
 
     {
-      int pos = -1;
+      int info_ok, pos = -1, cur_pos = 0, cur_time = 0, length = 0;
+
+      info_ok = xine_get_pos_length (gui->stream, &cur_pos, &cur_time, &length);
+      if (info_ok) {
+        pthread_mutex_lock (&gui->seek_mutex);
+        gui->stream_length.pos = cur_pos;
+        gui->stream_length.time = cur_time;
+        gui->stream_length.length = length;
+        pthread_mutex_unlock (&gui->seek_mutex);
+      }
 
       while (1) {
         pthread_mutex_lock (&gui->seek_mutex);
@@ -1272,34 +1205,52 @@ static void *gui_seek_thread (void *data) {
           break;
         }
         if (pos != gui->seek_pos) {
+          int ret;
 
           pos = gui->seek_pos;
           pthread_mutex_unlock (&gui->seek_mutex);
+
           /* panel_update_slider (gui->panel, pos); */
           osd_stream_position (gui, pos);
-          _gui_xine_play (gui, gui->stream, pos, 0, update_mmk);
-          xine_get_pos_length (gui->stream,
-            &gui->stream_length.pos, &gui->stream_length.time, &gui->stream_length.length);
-          panel_update_runtime_display (gui->panel);
+          ret = _gui_xine_play (gui, gui->stream, pos, 0, update_mmk);
+          info_ok = xine_get_pos_length (gui->stream, &cur_pos, &cur_time, &length);
+          if (info_ok) {
+            pthread_mutex_lock (&gui->seek_mutex);
+            gui->stream_length.pos = cur_pos;
+            gui->stream_length.time = cur_time;
+            gui->stream_length.length = length;
+            pthread_mutex_unlock (&gui->seek_mutex);
+            panel_update_runtime_display (gui->panel);
+            if (ret == 2)
+              osd_update_status (gui);
+          }
 
         } else {
 
-          int cur_time;
           int timestep = gui->seek_timestep;
           gui->seek_timestep = 0;
           pthread_mutex_unlock (&gui->seek_mutex);
+
           if ( xine_get_stream_info (gui->stream, XINE_STREAM_INFO_SEEKABLE) &&
-              (xine_get_status (gui->stream) == XINE_STATUS_PLAY) &&
-               xine_get_pos_length (gui->stream, NULL, &cur_time, NULL)) {
+              (xine_get_status (gui->stream) == XINE_STATUS_PLAY)) {
+            int ret;
+
             gui->ignore_next = 1;
             cur_time += timestep;
             if (cur_time < 0)
               cur_time = 0;
-            _gui_xine_play (gui, gui->stream, 0, cur_time, 1);
-            if (xine_get_pos_length (gui->stream,
-              &gui->stream_length.pos, &gui->stream_length.time, &gui->stream_length.length)) {
+            ret = _gui_xine_play (gui, gui->stream, 0, cur_time, 1);
+            info_ok = xine_get_pos_length (gui->stream, &cur_pos, &cur_time, &length);
+            if (info_ok) {
+              pthread_mutex_lock (&gui->seek_mutex);
+              gui->stream_length.pos = cur_pos;
+              gui->stream_length.time = cur_time;
+              gui->stream_length.length = length;
+              pthread_mutex_unlock (&gui->seek_mutex);
               panel_update_slider (gui->panel, gui->stream_length.pos);
               osd_stream_position (gui, gui->stream_length.pos);
+              if (ret == 2)
+                osd_update_status (gui);
             }
             gui->ignore_next = 0;
           }
@@ -1309,7 +1260,7 @@ static void *gui_seek_thread (void *data) {
     }
 
     gui->ignore_next = 0;
-    osd_hide_status (gui);
+    /* osd_hide_status (gui); */
     panel_check_pause (gui->panel);
 
     return NULL;
@@ -1374,131 +1325,31 @@ void gui_seek_relative (gGui_t *gui, int off_sec) {
 
 void gui_dndcallback (void *_gui, const char *filename) {
   gGui_t *gui = _gui;
-  int   more_than_one = -2;
-  char *mrl           = filename ? strdup(filename) : NULL;
+  int n;
 
-  if(mrl) {
-    char  buffer[strlen(mrl) + 10];
-    char  buffer2[strlen(mrl) + 10];
-    char *p;
+  if (!filename)
+    return;
 
-    memset(&buffer, 0, sizeof(buffer));
-    memset(&buffer2, 0, sizeof(buffer2));
+  n = gui_playlist_add_item (gui, filename, GUI_MAX_DIR_LEVELS, GUI_ITEM_TYPE_AUTO, 0);
+  if (!n)
+    return;
 
-    pthread_mutex_lock(&gui->mmk_mutex);
+  playlist_update_playlist (gui);
 
-    if((strlen(mrl) > 6) &&
-       (!strncmp(mrl, "file:", 5))) {
-
-      if((p = strstr(mrl, ":/")) != NULL) {
-	struct stat pstat;
-
-	p += 2;
-
-	if(*(p + 1) == '/')
-	  p++;
-
-      __second_stat:
-
-	if((stat(p, &pstat)) == 0) {
-	  if(is_a_dir(p)) {
-	    if(*(p + (strlen(p) - 1)) == '/')
-	      *(p + (strlen(p) - 1)) = '\0';
-
-            mediamark_collect_from_directory (gui, p);
-	    more_than_one = gui->playlist.cur;
-	    goto __do_play;
-	  }
-	  else
-	    snprintf(buffer, sizeof(buffer), "file:/%s", p);
-	}
-	else {
-	  snprintf(buffer2, sizeof(buffer2), "/%s", p);
-
-	  /* file don't exist, add it anyway */
-	  if((stat(buffer2, &pstat)) == -1)
-	    strlcpy(buffer, mrl, sizeof(buffer));
-	  else {
-	    if(is_a_dir(buffer2)) {
-
-	      if(buffer2[strlen(buffer2) - 1] == '/')
-		buffer2[strlen(buffer2) - 1] = '\0';
-
-              mediamark_collect_from_directory (gui, buffer2);
-	      more_than_one = gui->playlist.cur;
-	      goto __do_play;
-	    }
-	    else
-	      snprintf(buffer, sizeof(buffer), "file:/%s", buffer2);
-	  }
-
-	}
-      }
-      else {
-	p = mrl + 5;
-	goto __second_stat;
-      }
+  if (!(gui->playlist.control & PLAYLIST_CONTROL_IGNORE)) {
+    if ((xine_get_status (gui->stream) == XINE_STATUS_STOP) || gui->logo_mode) {
+      gui_playlist_lock (gui);
+      gui->playlist.cur = gui->playlist.num - n;
+      gui_playlist_unlock (gui);
+      gui_current_set_index (gui, GUI_MMK_CURRENT);
+      gui_pl_updated (gui);
+      if (gui->smart_mode)
+        gui_play (NULL, gui);
     }
-    else
-      strlcpy(buffer, mrl, sizeof(buffer));
-
-    if(is_a_dir(buffer)) {
-      if(buffer[strlen(buffer) - 1] == '/')
-	buffer[strlen(buffer) - 1] = '\0';
-
-      mediamark_collect_from_directory (gui, buffer);
-      more_than_one = gui->playlist.cur;
-    }
-    else {
-      char *ident;
-
-      /* If possible, use only base name as identifier to better fit into the display */
-      if((ident = strrchr(buffer, '/')) && ident[1])
-	ident++;
-      else
-	ident = buffer;
-
-      if(mrl_look_like_playlist(buffer)) {
-	int cur = gui->playlist.cur;
-
-	more_than_one = (gui->playlist.cur - 1);
-        if (mediamark_concat_mediamarks (gui, buffer))
-	  gui->playlist.cur = cur;
-	else
-          mediamark_append_entry (gui, buffer, ident, NULL, 0, -1, 0, 0);
-      }
-      else
-        mediamark_append_entry (gui, buffer, ident, NULL, 0, -1, 0, 0);
-
-    }
-
-  __do_play:
-
-    playlist_update_playlist (gui);
-
-    if(!(gui->playlist.control & PLAYLIST_CONTROL_IGNORE)) {
-
-      if((xine_get_status(gui->stream) == XINE_STATUS_STOP) || gui->logo_mode) {
-	if((more_than_one > -2) && ((more_than_one + 1) < gui->playlist.num))
-	  gui->playlist.cur = more_than_one + 1;
-	else
-	  gui->playlist.cur = gui->playlist.num - 1;
-
-        set_mmk (gui, mediamark_get_current_mmk (gui));
-        mmk_set_update(gui);
-	if(gui->smart_mode)
-          gui_play (NULL, gui);
-
-      }
-    }
-
-    if ((!is_playback_widgets_enabled (gui->panel)) && gui->playlist.num)
-      enable_playback_controls (gui->panel, 1);
-
-    pthread_mutex_unlock(&gui->mmk_mutex);
   }
 
-  free(mrl);
+  if ((!is_playback_widgets_enabled (gui->panel)) && gui->playlist.num)
+    enable_playback_controls (gui->panel, 1);
 }
 
 void gui_step_mrl (gGui_t *gui, int by) {
@@ -1549,7 +1400,7 @@ void gui_step_mrl (gGui_t *gui, int by) {
 
 	  gui->ignore_next = 1;
 	  gui->playlist.cur = newcur;
-          gui_set_current_mmk (gui, mediamark_get_current_mmk (gui));
+          gui_current_set_index (gui, GUI_MMK_CURRENT);
           if (!gui_xine_open_and_play (gui, gui->mmk.mrl, gui->mmk.sub, 0,
 				     gui->mmk.start, gui->mmk.av_offset, gui->mmk.spu_offset, 1))
             gui_display_logo (gui);
@@ -1587,7 +1438,7 @@ void gui_step_mrl (gGui_t *gui, int by) {
 	  gui->ignore_next = 1;
           gui->playlist.cur -= by;
 	  if((gui->playlist.cur < gui->playlist.num)) {
-            gui_set_current_mmk (gui, mediamark_get_current_mmk (gui));
+            gui_current_set_index (gui, GUI_MMK_CURRENT);
             if (!gui_xine_open_and_play (gui, gui->mmk.mrl, gui->mmk.sub, 0,
 				       gui->mmk.start, gui->mmk.av_offset, gui->mmk.spu_offset, 1))
               gui_display_logo (gui);
@@ -1604,7 +1455,7 @@ void gui_step_mrl (gGui_t *gui, int by) {
         if ((gui->playlist.cur - by) >= 0) {
           gui->playlist.cur -= by;
 	  if((gui->playlist.cur < gui->playlist.num)) {
-            gui_set_current_mmk (gui, mediamark_get_current_mmk (gui));
+            gui_current_set_index (gui, GUI_MMK_CURRENT);
             if (!gui_xine_open_and_play (gui, gui->mmk.mrl, gui->mmk.sub, 0,
 				       gui->mmk.start, gui->mmk.av_offset, gui->mmk.spu_offset, 1))
               gui_display_logo (gui);
@@ -1617,7 +1468,7 @@ void gui_step_mrl (gGui_t *gui, int by) {
           int newcur = (gui->playlist.cur - by) + gui->playlist.num;
 
 	  gui->playlist.cur = newcur;
-          gui_set_current_mmk (gui, mediamark_get_current_mmk (gui));
+          gui_current_set_index (gui, GUI_MMK_CURRENT);
           if (!gui_xine_open_and_play (gui, gui->mmk.mrl, gui->mmk.sub, 0,
 				     gui->mmk.start, gui->mmk.av_offset, gui->mmk.spu_offset, 1))
             gui_display_logo (gui);
@@ -1681,23 +1532,6 @@ void gui_mrlbrowser_show(xitk_widget_t *w, void *data) {
 
 }
 
-void gui_set_current_mmk (gGui_t *gui, mediamark_t *mmk) {
-  pthread_mutex_lock(&gui->mmk_mutex);
-  set_mmk(gui, mmk);
-  pthread_mutex_unlock(&gui->mmk_mutex);
-
-  mmk_set_update(gui);
-}
-
-void gui_set_current_mmk_by_index (gGui_t *gui, int idx) {
-  pthread_mutex_lock(&gui->mmk_mutex);
-  set_mmk(gui, mediamark_get_mmk_by_index (gui, idx));
-  gui->playlist.cur = idx;
-  pthread_mutex_unlock(&gui->mmk_mutex);
-
-  mmk_set_update(gui);
-}
-
 void gui_acontrol_show(xitk_widget_t *w, void *data) {
   gGui_t *gui = data;
 
@@ -1971,7 +1805,7 @@ void gui_add_mediamark (gGui_t *gui) {
     if (gui_xine_get_pos_length (gui, gui->stream, NULL, &secs, NULL)) {
       secs /= 1000;
 
-      mediamark_append_entry (gui, gui->mmk.mrl, gui->mmk.ident,
+      gui_playlist_append (gui, gui->mmk.mrl, gui->mmk.ident,
 			     gui->mmk.sub, secs, -1, gui->mmk.av_offset, gui->mmk.spu_offset);
       playlist_update_playlist (gui);
     }
@@ -2011,24 +1845,15 @@ static void fileselector_callback (filebrowser_t *fb, void *userdata) {
   free(cur_dir);
 
   /* Get the file path/name */
-  if(((file = filebrowser_get_full_filename(fb)) != NULL) && strlen(file)) {
+  file = filebrowser_get_full_filename (fb);
+  if (file && file[0]) {
     int first  = gui->playlist.num;
-    char *ident;
-
-    /* Get the name only to use as an identifier in the playlist display */
-    ident = filebrowser_get_current_filename(fb);
 
     /* If the file has an extension which could be a playlist, attempt to append
        it to the current list as a list; otherwise, append it as an ordinary file. */
-    if(mrl_look_like_playlist(file)) {
-      if (!mediamark_concat_mediamarks (gui, file))
-        mediamark_append_entry (gui, file, ident, NULL, 0, -1, 0, 0);
-    }
-    else
-      mediamark_append_entry (gui, file, ident, NULL, 0, -1, 0, 0);
+    gui_playlist_add_item (gui, file, 1, GUI_ITEM_TYPE_AUTO, 0);
 
     playlist_update_playlist (gui);
-    free(ident);
 
     /* Enable controls on display */
     if ((!is_playback_widgets_enabled (gui->panel)) && gui->playlist.num)
@@ -2040,7 +1865,7 @@ static void fileselector_callback (filebrowser_t *fb, void *userdata) {
       (gui->logo_mode || (xine_get_status(gui->stream) == XINE_STATUS_STOP))) {
       gui->playlist.cur = first;
       if(gui->smart_mode) {
-        gui_set_current_mmk (gui, mediamark_get_current_mmk (gui));
+        gui_current_set_index (gui, GUI_MMK_CURRENT);
         gui_play (NULL, gui);
       }
     }
@@ -2055,46 +1880,31 @@ static void fileselector_callback (filebrowser_t *fb, void *userdata) {
    currently selected directory to the current playlist. */
 static void fileselector_all_callback (filebrowser_t *fb, void *userdata) {
   gGui_t *gui = userdata;
-  char **files;
-  char  *path = filebrowser_get_current_dir(fb);
+  char *path = filebrowser_get_current_dir (fb);
 
-  /* Update the configuration with the current path */
-  if(path && strlen(path)) {
-    strlcpy(gui->curdir, path, sizeof(gui->curdir));
+  if (path && path[0]) {
+    char **files;
+    /* Update the configuration with the current path */
+    strlcpy (gui->curdir, path, sizeof (gui->curdir));
     config_update_string (gui->xine, "media.files.origin_path", gui->curdir);
-  }
-
-  /* Get all of the file names in the current directory as an array of pointers to strings */
-  if((files = filebrowser_get_all_files(fb)) != NULL) {
-    int i = 0;
-
-    if(path && strlen(path)) {
-      char pathname[XITK_PATH_MAX + 1 + 1]; /* +1 for trailing '/' */
-      char fullfilename[XITK_PATH_MAX + XITK_NAME_MAX + 2];
-      int  first = gui->playlist.num; /* current count of entries in playlist */
-
-      /* If the path is anything other than "/", append a slash to it so that it can
-         be concatenated with the file name */
-      if(strcasecmp(path, "/"))
-        snprintf(pathname, sizeof(pathname), "%s/", path);
-      else
-	strlcpy(pathname, path, sizeof(pathname));
-
-      /* For each file, concatenate the path with the name and append it to the playlist */
-      while(files[i]) {
-        snprintf(fullfilename, sizeof(fullfilename), "%s%s", pathname, files[i]);
+    /* Get all of the file names in the current directory as an array of pointers to strings */
+    files = filebrowser_get_all_files (fb);
+    if (files && files[0]) {
+      char buf[2048], *add, *e = buf + sizeof (buf);
+      int i, first = gui->playlist.num; /* current count of entries in playlist */
 
+      add = buf + strlcpy (buf, path, e - buf - 2);
+      if (add > e - 3)
+        add = e - 3;
+      if (add[-1] != '/')
+        *add++ = '/';
+      for (i = 0; files[i]; i++) {
+        strlcpy (add, files[i], e - add);
+        free (files[i]);
         /* If the file has an extension which could be a playlist, attempt to append
            it to the current list as a list; otherwise, append it as an ordinary file. */
-        if(mrl_look_like_playlist(fullfilename)) {
-          if (!mediamark_concat_mediamarks (gui, fullfilename))
-            mediamark_append_entry (gui, fullfilename, files[i], NULL, 0, -1, 0, 0);
-        }
-        else
-          mediamark_append_entry (gui, fullfilename, files[i], NULL, 0, -1, 0, 0);
-
-        i++;
-      } /* End while */
+        gui_playlist_add_item (gui, buf, 1, GUI_ITEM_TYPE_AUTO, 0);
+      }
 
       playlist_update_playlist (gui);
 
@@ -2107,20 +1917,14 @@ static void fileselector_all_callback (filebrowser_t *fb, void *userdata) {
       if((first != gui->playlist.num) && (xine_get_status(gui->stream) == XINE_STATUS_STOP)) {
         gui->playlist.cur = first;
         if(gui->smart_mode) {
-          gui_set_current_mmk (gui, mediamark_get_current_mmk (gui));
+          gui_current_set_index (gui, GUI_MMK_CURRENT);
           gui_play (NULL, gui);
         }
       }
-    } /* If valid path */
-
-    i = 0;
-    while(files[i])
-      free(files[i++]);
-
-    free(files);
-  } /* If valid file list */
-
-  free(path);
+    }
+    free (files);
+  }
+  free (path);
 
   gui->load_stream = NULL;
 }
@@ -2154,36 +1958,24 @@ static void subselector_callback (filebrowser_t *fb, void *userdata) {
   char *file;
   int ret;
 
-  if((file = filebrowser_get_full_filename(fb)) != NULL) {
-    if(file) {
-      mediamark_t *mmk = mediamark_clone_mmk(mediamark_get_current_mmk (gui));
+  file = filebrowser_get_full_filename (fb);
+  if (file) {
+    mediamark_set_str_val (&gui->playlist.mmk[gui->playlist.cur], file, MMK_VAL_SUB);
+    gui_current_set_index (gui, GUI_MMK_CURRENT);
+    playlist_mrlident_toggle (gui);
 
-      if(mmk) {
-	mediamark_replace_entry(&gui->playlist.mmk[gui->playlist.cur], mmk->mrl, mmk->ident,
-				file, mmk->start, mmk->end, mmk->av_offset, mmk->spu_offset);
-	mediamark_free_mmk(&mmk);
-
-        mmk = mediamark_get_current_mmk (gui);
-        gui_set_current_mmk (gui, mmk);
-
-        playlist_mrlident_toggle (gui);
-
-	if(xine_get_status(gui->stream) == XINE_STATUS_PLAY) {
-	  int curpos;
-	  xine_close (gui->spu_stream);
-
-	  gui_messages_off (gui);
-	  ret = xine_open(gui->spu_stream, mmk->sub);
-	  gui_messages_on (gui);
-	  if (ret)
-	    xine_stream_master_slave(gui->stream,
-				     gui->spu_stream, XINE_MASTER_SLAVE_PLAY | XINE_MASTER_SLAVE_STOP);
-
-          if (gui_xine_get_pos_length (gui, gui->stream, &curpos, NULL, NULL)) {
-	    xine_stop(gui->stream);
-            gui_set_current_position (gui, curpos);
-	  }
-	}
+    if (xine_get_status (gui->stream) == XINE_STATUS_PLAY) {
+      int curpos;
+      xine_close (gui->spu_stream);
+      gui_messages_off (gui);
+      ret = xine_open (gui->spu_stream, file);
+      gui_messages_on (gui);
+      if (ret)
+        xine_stream_master_slave (gui->stream,
+          gui->spu_stream, XINE_MASTER_SLAVE_PLAY | XINE_MASTER_SLAVE_STOP);
+      if (gui_xine_get_pos_length (gui, gui->stream, &curpos, NULL, NULL)) {
+        xine_stop (gui->stream);
+        gui_set_current_position (gui, curpos);
       }
     }
     free(file);
