@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2021 the xine project
+ * Copyright (C) 2000-2022 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -35,6 +35,7 @@
 typedef struct gGui_st gGui_t;
 
 #include "xitk.h"
+#include "dlist.h"
 
 typedef struct xui_keyedit_s xui_keyedit_t;
 typedef struct xui_vwin_st xui_vwin_t;
@@ -136,6 +137,17 @@ typedef enum {
 
 typedef struct skins_locations_s skins_locations_t;
 
+typedef enum {
+  XUI_EXTS_ANY = 0,
+  XUI_EXTS_MEDIA,
+  XUI_EXTS_AUDIO,
+  XUI_EXTS_VIDEO,
+  XUI_EXTS_IMAGE,
+  XUI_EXTS_SUBTITLE,
+  XUI_EXTS_PLAYLIST,
+  XUI_EXTS_LAST
+} xui_exts_t;
+
 struct gGui_st {
   struct gGui_st           *nextprev[3];
 
@@ -196,15 +208,18 @@ struct gGui_st {
   post_info_t               post_audio, post_video;
 
   struct {
+    pthread_mutex_t         mutex;
+
     int                     enabled;
     int                     use_unscaled;
     int                     unscaled_available;
     int                     timeout;
 
-    osd_object_t            sinfo;
-    osd_object_t            bar;
-    osd_object_t            status;
-    osd_object_t            info;
+    xitk_dlist_t            pending;
+    struct xui_osd_obj_s {
+      xitk_dnode_t          node;
+      osd_object_t          obj;
+    }                       sinfo, bar, status, info;
 
   } osd;
 
@@ -247,11 +262,14 @@ struct gGui_st {
   struct {
     mediamark_t           **mmk;
 
-    int                     num;                   /* number of entries in playlist */
-    int                     cur;                   /* current entry in playlist */
-    int                     ref_append;            /* append mrl reference entry to this position */
-    int                     loop;                  /* current loop mode (see PLAYLIST_LOOP_* */
-    int                     control;               /* see PLAYLIST_CONTROL_* */
+    int                     max;        /** << number of entries that fit .mmk */
+    int                     num;        /** << number of entries set in .mmk */
+    int                     cur;        /** << current entry in playlist */
+    int                     ref_append; /** << append mrl reference entry to this position */
+    int                     loop;       /** << see PLAYLIST_LOOP_* */
+    int                     control;    /** << see PLAYLIST_CONTROL_* */
+
+    xine_sarray_t          *exts[XUI_EXTS_LAST];
   } playlist;
 
 
@@ -269,6 +287,11 @@ struct gGui_st {
     int                     method;
   } mixer;
 
+  struct {
+    const char             *msg[4];
+    int                     flags;
+  } transform;
+
   int                       layer_above;
   int                       always_layer_above;
 
@@ -404,6 +427,13 @@ void reparent_window(gGui_t *gui, xitk_window_t *xwin);
 void gui_load_window_pos (gGui_t *gui, const char *name, int *x, int *y);
 void gui_save_window_pos (gGui_t *gui, const char *name, xitk_register_key_t key);
 
+void gui_playlist_init (gGui_t *gui);
+void gui_playlist_lock (gGui_t *gui);
+#define gui_playlist_lock(_gui) pthread_mutex_lock (&(_gui)->mmk_mutex)
+void gui_playlist_unlock (gGui_t *gui);
+#define gui_playlist_unlock(_gui) pthread_mutex_unlock (&(_gui)->mmk_mutex)
+void gui_playlist_deinit (gGui_t *gui);
+
 #ifdef HAVE_XML_PARSER_REENTRANT
 # define xml_parser_init_R(X,D,L,M) X = xml_parser_init_r ((D), (L), (M))
 # define xml_parser_build_tree_R(X,T) xml_parser_build_tree_r ((X), (T))
@@ -415,3 +445,4 @@ void gui_save_window_pos (gGui_t *gui, const char *name, xitk_register_key_t key
 #endif
 
 #endif
+
