@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2021 the xine project
+ * Copyright (C) 2000-2022 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -347,9 +347,6 @@ static int vctrl_open_window (xui_vctrl_t *vctrl) {
   xitk_image_t              *bg_image;
   int x, y, width, height;
 
-  XITK_WIDGET_INIT (&br);
-  XITK_WIDGET_INIT (&lb);
-
   {
     const xitk_skin_element_info_t *info = xitk_skin_get_info (vctrl->gui->skin_config, "CtlBG");
     bg_image = info ? info->pixmap_img.image : NULL;
@@ -374,7 +371,8 @@ static int vctrl_open_window (xui_vctrl_t *vctrl) {
    */
   vctrl->widget_list = xitk_window_widget_list(vctrl->xwin);
 
-  XITK_WIDGET_INIT (&lbl);
+  lbl.nw.wl       = vctrl->widget_list;
+  lbl.nw.userdata = vctrl;
   lbl.callback = NULL;
 
   {
@@ -389,7 +387,7 @@ static int vctrl_open_window (xui_vctrl_t *vctrl) {
     xitk_slider_widget_t sl;
     unsigned int u;
 
-    XITK_WIDGET_INIT (&sl);
+    sl.nw.wl = vctrl->widget_list;
     sl.min    = CONTROL_MIN;
     sl.max    = CONTROL_MAX;
     sl.step   = CONTROL_STEP;
@@ -406,30 +404,32 @@ static int vctrl_open_window (xui_vctrl_t *vctrl) {
       else
         v = CONTROL_MIN;
       */
-      sl.skin_element_name = sl_skins[u];
-      sl.userdata        =
-      sl.motion_userdata = item;
-      item->w = xitk_slider_create (vctrl->widget_list, vctrl->gui->skin_config, &sl);
+      sl.nw.skin_element_name = sl_skins[u];
+      sl.nw.userdata = item;
+      item->w = xitk_slider_create (&sl, vctrl->gui->skin_config);
       if (item->w) {
         xitk_add_widget (vctrl->widget_list, item->w, (item->enable ? XITK_WIDGET_STATE_ENABLE : 0) | XITK_WIDGET_STATE_VISIBLE);
         xitk_slider_set_pos (item->w, item->value);
         xitk_set_widget_tips (item->w, item->hint);
       }
-      lbl.skin_element_name = lbl_skins[u];
-      lbl.label             = item->label;
-      xitk_add_widget (vctrl->widget_list, xitk_label_create (vctrl->widget_list, vctrl->gui->skin_config, &lbl),
+      lbl.nw.skin_element_name = lbl_skins[u];
+      lbl.label = item->label;
+      xitk_add_widget (vctrl->widget_list, xitk_label_create (&lbl, vctrl->gui->skin_config),
         XITK_WIDGET_STATE_ENABLE | XITK_WIDGET_STATE_VISIBLE);
     }
   }
 
   vctrl->skins_num = skin_get_names (vctrl->gui, vctrl->skins, sizeof (vctrl->skins) / sizeof (vctrl->skins[0]));
 
-  lbl.skin_element_name = "CtlSkLbl";
+  lbl.nw.skin_element_name = "CtlSkLbl";
   /* TRANSLATORS: only ASCII characters (skin) */
   lbl.label             = pgettext("skin", "Choose a Skin");
-  xitk_add_widget (vctrl->widget_list, xitk_label_create (vctrl->widget_list, vctrl->gui->skin_config, &lbl),
+  xitk_add_widget (vctrl->widget_list, xitk_label_create (&lbl, vctrl->gui->skin_config),
     XITK_WIDGET_STATE_ENABLE | XITK_WIDGET_STATE_VISIBLE);
 
+  br.nw.wl                         = vctrl->widget_list;
+  br.nw.skin_element_name          = NULL;
+  br.nw.userdata                   = vctrl;
   br.arrow_up.skin_element_name    = "CtlSkUp";
   br.slider.skin_element_name      = "SliderCtlSk";
   br.arrow_dn.skin_element_name    = "CtlSkDn";
@@ -441,20 +441,20 @@ static int vctrl_open_window (xui_vctrl_t *vctrl) {
   br.browser.entries               = vctrl->skins;
   br.callback                      =
   br.dbl_click_callback            = control_select_new_skin;
-  br.userdata                      = vctrl;
-  vctrl->skinlist = xitk_browser_create (vctrl->widget_list, vctrl->gui->skin_config, &br);
+  vctrl->skinlist = xitk_browser_create (&br, vctrl->gui->skin_config);
   xitk_add_widget (vctrl->widget_list, vctrl->skinlist, XITK_WIDGET_STATE_ENABLE | XITK_WIDGET_STATE_VISIBLE);
 
   xitk_browser_update_list (vctrl->skinlist, vctrl->skins, NULL, vctrl->skins_num, 0);
 
-  lb.skin_element_name = "CtlDismiss";
-  lb.button_type       = CLICK_BUTTON;
-  lb.align             = ALIGN_DEFAULT;
-  lb.label             = _("Dismiss");
-  lb.callback          = _control_toggle_window;
-  lb.state_callback    = NULL;
-  lb.userdata          = vctrl;
-  w = xitk_labelbutton_create (vctrl->widget_list, vctrl->gui->skin_config, &lb);
+  lb.nw.wl                = vctrl->widget_list;
+  lb.nw.skin_element_name = "CtlDismiss";
+  lb.nw.userdata          = vctrl;
+  lb.button_type    = CLICK_BUTTON;
+  lb.align          = ALIGN_DEFAULT;
+  lb.label          = _("Dismiss");
+  lb.callback       = _control_toggle_window;
+  lb.state_callback = NULL;
+  w = xitk_labelbutton_create (&lb, vctrl->gui->skin_config);
   xitk_add_widget (vctrl->widget_list, w, XITK_WIDGET_STATE_ENABLE | XITK_WIDGET_STATE_VISIBLE);
   xitk_set_widget_tips(w, _("Close control window"));
 
