@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2021 the xine project
+ * Copyright (C) 2000-2022 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -176,9 +176,7 @@ static int event_sender_event (void *data, const xitk_be_event_t *e) {
             case XITK_KEY_DOWN:  xine_event.type = XINE_EVENT_INPUT_DOWN; break;
             case XITK_KEY_LEFT:  xine_event.type = XINE_EVENT_INPUT_LEFT; break;
             case XITK_KEY_RIGHT: xine_event.type = XINE_EVENT_INPUT_RIGHT; break;
-            case XITK_KEY_RETURN:
-            case XITK_KEY_NUMPAD_ENTER:
-            case XITK_KEY_ISO_ENTER: xine_event.type = XINE_EVENT_INPUT_SELECT; break;
+            case XITK_KEY_RETURN: xine_event.type = XINE_EVENT_INPUT_SELECT; break;
             case XITK_KEY_ESCAPE:
               event_sender_exit (NULL, es, 0);
               return 1;
@@ -351,18 +349,19 @@ void event_sender_panel (gGui_t *gui) {
   {
     xitk_labelbutton_widget_t lb;
     int i;
-    XITK_WIDGET_INIT (&lb);
+
+    lb.nw.wl                = es->widget_list;
+    lb.nw.skin_element_name = NULL;
 
     lb.button_type       = CLICK_BUTTON;
     lb.align             = ALIGN_CENTER;
     lb.callback          = event_sender_send2;
     lb.state_callback    = NULL;
-    lb.skin_element_name = NULL;
 
     for (i = 0; i < 18; i++) {
+      lb.nw.userdata = es->self + i;
       lb.label    = _es_event_types[i].label;
-      lb.userdata = es->self + i;
-      es->wdgts[i] = xitk_noskin_labelbutton_create (es->widget_list, &lb,
+      es->wdgts[i] = xitk_noskin_labelbutton_create (&lb,
         _es_event_types[i].x, _es_event_types[i].y,
         _es_event_types[i].w, _es_event_types[i].h,
         XITK_NOSKIN_TEXT_NORM, XITK_NOSKIN_TEXT_NORM, XITK_NOSKIN_TEXT_INV, hboldfontname);
@@ -370,9 +369,9 @@ void event_sender_panel (gGui_t *gui) {
     }
 
     for (; i < (int)(sizeof (_es_event_types) / sizeof (_es_event_types[0])); i++) {
+      lb.nw.userdata = es->self + i;
       lb.label    = gettext (_es_event_types[i].label);
-      lb.userdata = es->self + i;
-      es->wdgts[i] = xitk_noskin_labelbutton_create (es->widget_list, &lb,
+      es->wdgts[i] = xitk_noskin_labelbutton_create (&lb,
         _es_event_types[i].x, _es_event_types[i].y,
         _es_event_types[i].w, _es_event_types[i].h,
         XITK_NOSKIN_TEXT_NORM, XITK_NOSKIN_TEXT_NORM, XITK_NOSKIN_TEXT_INV, hboldfontname);
@@ -383,11 +382,11 @@ void event_sender_panel (gGui_t *gui) {
 
     x = WINDOW_WIDTH - 70 - 5;
     y = WINDOW_HEIGHT - 23 - 5;
+    lb.nw.userdata = es;
     lb.label    = _("Close");
     lb.callback = event_sender_exit;
-    lb.userdata = es;
-    w = xitk_noskin_labelbutton_create (es->widget_list,
-      &lb, x, y, 70, 23, XITK_NOSKIN_TEXT_NORM, XITK_NOSKIN_TEXT_NORM, XITK_NOSKIN_TEXT_INV, hboldfontname);
+    w = xitk_noskin_labelbutton_create (&lb,
+      x, y, 70, 23, XITK_NOSKIN_TEXT_NORM, XITK_NOSKIN_TEXT_NORM, XITK_NOSKIN_TEXT_INV, hboldfontname);
     xitk_add_widget (es->widget_list, w, XITK_WIDGET_STATE_ENABLE | XITK_WIDGET_STATE_VISIBLE);
   }
 
