@@ -1,4 +1,4 @@
-/** Copyright (C) 2000-2021 the xine project
+/** Copyright (C) 2000-2022 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -889,28 +889,20 @@ static void event_listener (void *user_data, const xine_event_t *event) {
 
     /* request title display change in ui */
   case XINE_EVENT_UI_SET_TITLE:
-    if(event->stream == gui->stream) {
+    if (event->stream == gui->stream) {
       xine_ui_data_t *uevent = (xine_ui_data_t *) event->data;
+      mediamark_t *m = &gui->mmk;
+      int changed;
 
-      pthread_mutex_lock (&gui->mmk_mutex);
-      if(strcmp(gui->mmk.ident, uevent->str)) {
-
-	free(gui->mmk.ident);
-	if(gui->playlist.num && gui->playlist.cur >= 0 && gui->playlist.mmk &&
-	   gui->playlist.mmk[gui->playlist.cur]) {
-
-	  free(gui->playlist.mmk[gui->playlist.cur]->ident);
-
-	  gui->playlist.mmk[gui->playlist.cur]->ident = strdup(uevent->str);
-	}
-	gui->mmk.ident = strdup(uevent->str);
-        pthread_mutex_unlock (&gui->mmk_mutex);
+      gui_playlist_set_str_val (gui, uevent->str, MMK_VAL_IDENT, GUI_MMK_CURRENT);
+      gui_playlist_lock (gui);
+      changed = mediamark_set_str_val (&m, uevent->str, MMK_VAL_IDENT);
+      gui_playlist_unlock (gui);
 
+      if (changed) {
         video_window_set_mrl (gui->vwin, uevent->str);
         playlist_mrlident_toggle (gui);
         panel_update_mrl_display (gui->panel);
-      } else {
-        pthread_mutex_unlock (&gui->mmk_mutex);
       }
     }
     break;
@@ -986,9 +978,9 @@ static void event_listener (void *user_data, const xine_event_t *event) {
           flags = _MSG_WITH_ARGS;
           {
             int i;
-            pthread_mutex_lock (&gui->mmk_mutex);
+            gui_playlist_lock (gui);
             i = strncasecmp (gui->mmk.mrl, "dvd:/", 5);
-            pthread_mutex_unlock (&gui->mmk_mutex);
+            gui_playlist_unlock (gui);
             if (!i)
               s2 = _("\nYour DVD is probably crypted. According to your country laws, you can or can't "
                      "install/use libdvdcss to be able to read this disc, which you bought.");
@@ -1129,20 +1121,14 @@ static void event_listener (void *user_data, const xine_event_t *event) {
 	printf("XINE_EVENT_MRL_REFERENCE got mrl [%s] (alternative=%d)\n",
                ref->mrl, ref->alternative);
 
+      gui_playlist_lock (gui);
       if(ref->alternative == 0) {
         gui->playlist.ref_append++;
-        mediamark_insert_entry(gui->playlist.ref_append, ref->mrl, ref->mrl, NULL, 0, -1, 0, 0);
+        gui_playlist_insert(gui->playlist.ref_append, ref->mrl, ref->mrl, NULL, 0, -1, 0, 0);
       } else {
-        pthread_mutex_lock (&gui->mmk_mutex);
-	mediamark_t *mmk = mediamark_get_mmk_by_index(gui->playlist.ref_append);
-
-	if(mmk) {
-	  mediamark_append_alternate_mrl(mmk, ref->mrl);
-	  mediamark_set_got_alternate(mmk);
-	}
-        pthread_mutex_unlock (&gui->mmk_mutex);
-
+        gui_playlist_set_str_val (gui, ref->mrl, MMK_VAL_ADD_ALTER, gui->playlist.ref_append);
       }
+      gui_playlist_unlock (gui);
 
     }
     break;
@@ -1167,24 +1153,19 @@ typedef struct {
 	printf("XINE_EVENT_MRL_REFERENCE_EXT got mrl [%s] (alternative=%d)\n",
                ref->mrl, ref->alternative);
 
+      gui_playlist_lock (gui);
       if(ref->alternative == 0) {
         gui->playlist.ref_append++;
         /* FIXME: duration handled correctly? */
-        mediamark_insert_entry (gui, gui->playlist.ref_append, ref->mrl, *title ? title : ref->mrl, NULL,
+        gui_playlist_insert (gui, gui->playlist.ref_append, ref->mrl, *title ? title : ref->mrl, NULL,
                                ref->start_time,
                                ref->duration ? (int)(ref->start_time + ref->duration) : -1,
                                0, 0);
       } else {
         /* FIXME: title? start? duration? */
-        pthread_mutex_lock (&gui->mmk_mutex);
-        mediamark_t *mmk = mediamark_get_mmk_by_index (gui, gui->playlist.ref_append);
-
-	if(mmk) {
-	  mediamark_append_alternate_mrl(mmk, ref->mrl);
-	  mediamark_set_got_alternate(mmk);
-	}
-        pthread_mutex_unlock (&gui->mmk_mutex);
+        gui_playlist_set_str_val (gui, ref->mrl, MMK_VAL_ADD_ALTER, gui->playlist.ref_append);
       }
+      gui_playlist_unlock (gui);
     }
     break;
 
@@ -1289,9 +1270,6 @@ int main(int argc, char *argv[]) {
   gui->network_port           = 0;
   gui->use_root_window        = 0;
   gui->actions_on_start[aos]  = ACTID_NOKEY;
-  gui->playlist.mmk           = NULL;
-  gui->playlist.loop          = PLAYLIST_LOOP_NO_LOOP;
-  gui->playlist.control       = 0;
   gui->skin_server_url        = NULL;
   gui->verbosity              = 0;
   gui->broadcast_port         = 0;
@@ -1351,6 +1329,8 @@ int main(int argc, char *argv[]) {
 
   visual_anim_init (gui);
 
+  gui_playlist_init (gui);
+
   /*
    * parse command line
    */
@@ -1565,12 +1545,7 @@ int main(int argc, char *argv[]) {
       if((!actions_on_start(gui->actions_on_start, ACTID_PLAYLIST) && (aos < MAX_ACTIONS_ON_START)))
 	gui->actions_on_start[aos++] = ACTID_PLAYLIST;
 
-      pthread_mutex_lock (&gui->mmk_mutex);
-      if(!gui->playlist.mmk)
-        mediamark_load_mediamarks (gui, optarg);
-      else
-        mediamark_concat_mediamarks (gui, optarg);
-      pthread_mutex_unlock (&gui->mmk_mutex);
+      gui_playlist_add_item (gui, optarg, 1, GUI_ITEM_TYPE_PLAYLIST, 0);
 
       /* don't load original playlist when loading this one */
       no_old_playlist = 1;
@@ -1882,7 +1857,7 @@ int main(int argc, char *argv[]) {
     char buffer[XITK_PATH_MAX + XITK_NAME_MAX + 2];
 
     snprintf(buffer, sizeof(buffer), "%s/.xine/xine-ui_old_playlist.tox", xine_get_homedir());
-    mediamark_load_mediamarks (gui, buffer);
+    gui_playlist_add_item (gui, buffer, 1, GUI_ITEM_TYPE_PLAYLIST, 0);
   }
 
   gui->subtitle_autoload =
@@ -2162,6 +2137,8 @@ int main(int argc, char *argv[]) {
 
   free(gui->keymap_file);
 
+  gui_playlist_deinit (gui);
+
   free (gui);
   return retval;
 }
@@ -2194,4 +2171,3 @@ void gui_save_window_pos (gGui_t *gui, const char *name, xitk_register_key_t key
     config_update_num (gui->xine, buf, wi.y);
   }
 }
-
