@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2021 the xine project
+ * Copyright (C) 2000-2022 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -460,14 +460,17 @@ static void panel_check_mute (xui_panel_t *panel) {
  */
 static void *slider_loop (void *data) {
   xui_panel_t *panel = data;
-  int screensaver_timer = 0, lastmsecs = -1, i = -1, step = 0;
+  int screensaver_timer = 0, lastmsecs = -1, it = 19, step = 0;
   struct timespec wtime = {0, 0};
 
   xitk_vers_string_init (&panel->sl.istr, NULL, 0);
   panel->sl.ident = stream_infos_ident_new ();
 
   while (1) {
-    i++;
+
+    if (--it < 0)
+      it = 19;
+
     if (step > 0) {
       wtime.tv_nsec += step * 1000000;
       if (wtime.tv_nsec >= 1000000000) {
@@ -541,7 +544,7 @@ static void *slider_loop (void *data) {
 
       }
 
-      if(!(i % 2)) {
+      if (it & 1) {
         osd_update (panel->gui);
 
         if (panel->gui->mrl_overrided) {
@@ -557,23 +560,19 @@ static void *slider_loop (void *data) {
         if (stream_infos_ident_get (panel->sl.ident, &panel->sl.istr, panel->gui->stream)) {
           if (!pthread_mutex_trylock (&panel->gui->mmk_mutex)) {
             if (panel->sl.istr.s && panel->sl.istr.s[0]) {
-              if (panel->gui->playlist.num && (panel->gui->playlist.cur >= 0) &&
-                panel->gui->playlist.mmk && panel->gui->playlist.mmk[panel->gui->playlist.cur] &&
-                (strcmp (panel->gui->mmk.ident, panel->sl.istr.s))) {
-                free (panel->gui->mmk.ident);
-                panel->gui->mmk.ident = strdup (panel->sl.istr.s);
-                free (panel->gui->playlist.mmk[panel->gui->playlist.cur]->ident);
-                panel->gui->playlist.mmk[panel->gui->playlist.cur]->ident = strdup (panel->sl.istr.s);
-                pthread_mutex_unlock (&panel->gui->mmk_mutex);
+              if (gui_playlist_set_str_val (panel->gui, panel->sl.istr.s, MMK_VAL_IDENT, GUI_MMK_CURRENT)) {
+                mediamark_t *m = &panel->gui->mmk;
+                mediamark_set_str_val (&m, panel->sl.istr.s, MMK_VAL_IDENT);
+                gui_playlist_unlock (panel->gui);
                 video_window_set_mrl (panel->gui->vwin, panel->sl.istr.s);
                 playlist_mrlident_toggle (panel->gui);
                 panel_update_mrl_display (panel);
               } else {
-                pthread_mutex_unlock (&panel->gui->mmk_mutex);
+                gui_playlist_unlock (panel->gui);
               }
             } else {
               video_window_set_mrl (panel->gui->vwin, panel->gui->mmk.mrl);
-              pthread_mutex_unlock (&panel->gui->mmk_mutex);
+              gui_playlist_unlock (panel->gui);
             }
           }
         }
@@ -581,7 +580,7 @@ static void *slider_loop (void *data) {
         {
           if (video_window_is_visible (panel->gui->vwin) > 1) {
             if (panel->gui->ssaver_timeout) {
-              if (!(i % 2))
+              if (!(it & 1))
                 screensaver_timer++;
               if (screensaver_timer >= panel->gui->ssaver_timeout)
                 screensaver_timer = video_window_reset_ssaver (panel->gui->vwin);
@@ -600,7 +599,7 @@ static void *slider_loop (void *data) {
 		xitk_slider_set_pos(panel->w[_W_slider_play], pos);
 	    }
 
-	    if(!(i % 20)) {
+            if (it == 0) {
 	      int level = 0;
 
               panel_update_channel_display (panel);
@@ -617,8 +616,6 @@ static void *slider_loop (void *data) {
 
 	      xitk_slider_set_pos (panel->w[_W_slider_mixer], level);
               panel_check_mute (panel);
-
-	      i = 0;
 	    }
 
 	  }
@@ -632,7 +629,7 @@ static void *slider_loop (void *data) {
     }
 
     if (panel->gui->cursor_visible) {
-      if(!(i % 2))
+      if (it & 1)
 	video_window_set_cursor_timer (panel->gui->vwin, video_window_get_cursor_timer (panel->gui->vwin) + 1);
 
       if (video_window_get_cursor_timer (panel->gui->vwin) >= 5) {
@@ -885,9 +882,9 @@ static void panel_spu_lang_list(xitk_widget_t *w, void *data) {
  */
 void panel_update_mrl_display (xui_panel_t *panel) {
   if (panel) {
-    pthread_mutex_lock (&panel->gui->mmk_mutex);
+    gui_playlist_lock (panel->gui);
     panel_set_title (panel, (panel->gui->is_display_mrl) ? panel->gui->mmk.mrl : panel->gui->mmk.ident);
-    pthread_mutex_unlock (&panel->gui->mmk_mutex);
+    gui_playlist_unlock (panel->gui);
   }
 }
 
@@ -949,9 +946,9 @@ void panel_snapshot (xitk_widget_t *w, void *data) {
   xui_panel_t *panel = data;
 
   (void)w;
-  pthread_mutex_lock (&panel->gui->mmk_mutex);
+  gui_playlist_lock (panel->gui);
   create_snapshot (panel->gui, panel->gui->mmk.mrl, panel_snapshot_error, panel_snapshot_info, panel->gui);
-  pthread_mutex_unlock (&panel->gui->mmk_mutex);
+  gui_playlist_unlock (panel->gui);
 }
 
 /*
@@ -1056,6 +1053,7 @@ void panel_add_autoplay_buttons (xui_panel_t *panel) {
       tips, XITK_TIPS_TIMEOUT_AUTO, 0);
     if (panel->autoplay_buttons)
       xitk_add_widget (panel->widget_list, panel->autoplay_buttons, XITK_WIDGET_STATE_KEEP);
+    xitk_set_widget_tips (panel->autoplay_buttons, _("More sources..."));
   }
 
   /* show panel (see panel_init ()) */
@@ -1224,138 +1222,138 @@ xui_panel_t *panel_init (gGui_t *gui) {
     xitk_widget_t *w;
     xitk_button_widget_t b;
 
-    XITK_WIDGET_INIT (&b);
+    b.nw.wl = panel->widget_list;
     b.state_callback    = NULL;
 
-    b.userdata          = GUI_PREV (panel->gui);
+    b.nw.userdata       = GUI_PREV (panel->gui);
 
-    b.skin_element_name = "Prev";
-    b.callback          = gui_nextprev_mrl;
-    panel->w[_W_prev] = xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "Prev";
+    b.callback = gui_nextprev_mrl;
+    panel->w[_W_prev] = xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, panel->w[_W_prev], XITK_WIDGET_STATE_KEEP);
 
-    b.skin_element_name = "AudioPrev";
-    b.callback          = gui_nextprev_audio_channel;
-    w = xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "AudioPrev";
+    b.callback = gui_nextprev_audio_channel;
+    w = xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, w, XITK_WIDGET_STATE_KEEP);
     xitk_set_widget_tips (w, _("Previous audio channel"));
 
-    b.skin_element_name = "SpuPrev";
-    b.callback          = gui_nextprev_spu_channel;
-    w = xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "SpuPrev";
+    b.callback = gui_nextprev_spu_channel;
+    w = xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, w, XITK_WIDGET_STATE_KEEP);
     xitk_set_widget_tips (w, _("Previous SPU channel"));
 
-    b.skin_element_name = "PlaySlow";
-    b.callback          = gui_nextprev_speed;
-    w = xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "PlaySlow";
+    b.callback = gui_nextprev_speed;
+    w = xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, w, XITK_WIDGET_STATE_KEEP);
     xitk_set_widget_tips (w, _("Slow motion"));
 
-    b.userdata          = GUI_NEXT (panel->gui);
+    b.nw.userdata = GUI_NEXT (panel->gui);
 
-    b.skin_element_name = "Next";
-    b.callback          = gui_nextprev_mrl;
-    panel->w[_W_next] = xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "Next";
+    b.callback = gui_nextprev_mrl;
+    panel->w[_W_next] = xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, panel->w[_W_next], XITK_WIDGET_STATE_KEEP);
 
-    b.skin_element_name = "AudioNext";
-    b.callback          = gui_nextprev_audio_channel;
-    w = xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "AudioNext";
+    b.callback = gui_nextprev_audio_channel;
+    w = xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, w, XITK_WIDGET_STATE_KEEP);
     xitk_set_widget_tips (w, _("Next audio channel"));
 
-    b.skin_element_name = "SpuNext";
-    b.callback          = gui_nextprev_spu_channel;
-    w = xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "SpuNext";
+    b.callback = gui_nextprev_spu_channel;
+    w = xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, w, XITK_WIDGET_STATE_KEEP);
     xitk_set_widget_tips (w, _("Next SPU channel"));
 
-    b.skin_element_name = "PlayFast";
-    b.callback          = gui_nextprev_speed;
-    w = xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "PlayFast";
+    b.callback = gui_nextprev_speed;
+    w = xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, w, XITK_WIDGET_STATE_KEEP);
     xitk_set_widget_tips (w, _("Fast motion"));
 
-    b.userdata          = panel->gui;
+    b.nw.userdata = panel->gui;
 
-    b.skin_element_name = "Stop";
-    b.callback          = gui_stop;
-    panel->w[_W_stop] =  xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "Stop";
+    b.callback = gui_stop;
+    panel->w[_W_stop] =  xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, panel->w[_W_stop], XITK_WIDGET_STATE_KEEP);
     xitk_set_widget_tips (panel->w[_W_stop], _("Stop playback"));
 
-    b.skin_element_name = "Play";
-    b.callback          = gui_play;
-    panel->w[_W_play] = xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "Play";
+    b.callback = gui_play;
+    panel->w[_W_play] = xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, panel->w[_W_play], XITK_WIDGET_STATE_KEEP);
     xitk_set_widget_tips (panel->w[_W_play], _("Play selected entry"));
 
-    b.skin_element_name = "Eject";
-    b.callback          = gui_eject;
-    panel->w[_W_eject] = xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "Eject";
+    b.callback = gui_eject;
+    panel->w[_W_eject] = xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, panel->w[_W_eject], XITK_WIDGET_STATE_KEEP);
     xitk_set_widget_tips (panel->w[_W_eject], _("Eject current medium"));
 
-    b.skin_element_name = "Exit";
-    b.callback          = gui_exit;
-    w = xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "Exit";
+    b.callback = gui_exit;
+    w = xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, w, XITK_WIDGET_STATE_KEEP);
     xitk_set_widget_tips (w, _("Quit"));
 
-    b.skin_element_name = "Setup";
-    b.callback          = gui_setup_show;
-    w = xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "Setup";
+    b.callback = gui_setup_show;
+    w = xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, w, XITK_WIDGET_STATE_KEEP);
     xitk_set_widget_tips (w, _("Setup window"));
 
-    b.skin_element_name = "Nav";
-    b.callback          = gui_event_sender_show;
-    w = xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "Nav";
+    b.callback = gui_event_sender_show;
+    w = xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, w, XITK_WIDGET_STATE_KEEP);
     xitk_set_widget_tips (w, _("Navigator"));
 
-    b.skin_element_name = "FullScreen";
-    b.callback          = gui_set_fullscreen_mode;
-    w = xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "FullScreen";
+    b.callback = gui_set_fullscreen_mode;
+    w = xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, w, XITK_WIDGET_STATE_KEEP);
     xitk_set_widget_tips (w, _("Fullscreen/Window mode"));
 
-    b.skin_element_name = "CtlBtn";
-    b.callback          = gui_control_show;
-    w = xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "CtlBtn";
+    b.callback = gui_control_show;
+    w = xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, w, XITK_WIDGET_STATE_KEEP);
     xitk_set_widget_tips (w, _("Control"));
 
-    b.skin_element_name = "AudioCtlBtn";
-    b.callback          = gui_acontrol_show;
-    w = xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "AudioCtlBtn";
+    b.callback = gui_acontrol_show;
+    w = xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, w, XITK_WIDGET_STATE_KEEP);
     xitk_set_widget_tips (w, _("Audio control"));
 
-    b.skin_element_name = "MrlBtn";
-    b.callback          = gui_mrlbrowser_show;
-    w =  xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "MrlBtn";
+    b.callback = gui_mrlbrowser_show;
+    w =  xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, w, XITK_WIDGET_STATE_KEEP);
     xitk_set_widget_tips (w, _("Open Location"));
 
-    b.skin_element_name = "PlBtn";
-    b.callback          = gui_playlist_show;
-    w =  xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "PlBtn";
+    b.callback = gui_playlist_show;
+    w =  xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, w, XITK_WIDGET_STATE_KEEP);
     xitk_set_widget_tips (w, _("Playlist"));
 
-    b.userdata          = panel;
+    b.nw.userdata = panel;
 
-    b.skin_element_name = "Close";
-    b.callback          = panel_toggle_visibility;
-    w =  xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "Close";
+    b.callback = panel_toggle_visibility;
+    w =  xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, w, XITK_WIDGET_STATE_KEEP);
     xitk_set_widget_tips (w, _("Hide GUI"));
 
-    b.skin_element_name = "Snapshot";
-    b.callback          = panel_snapshot;
-    w = xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "Snapshot";
+    b.callback = panel_snapshot;
+    w = xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, w, XITK_WIDGET_STATE_KEEP);
     xitk_set_widget_tips (w, _("Take a snapshot"));
   }
@@ -1369,80 +1367,81 @@ xui_panel_t *panel_init (gGui_t *gui) {
 
   {
     xitk_label_widget_t lbl;
-    XITK_WIDGET_INIT (&lbl);
-    lbl.userdata          = panel;
 
-    lbl.skin_element_name = "TimeLabel";
-    lbl.label             = "00:00:00";
-    lbl.callback          = _panel_change_time_label;
-    panel->runtime_label = xitk_label_create (panel->widget_list, panel->gui->skin_config, &lbl);
+    lbl.nw.wl       = panel->widget_list;
+    lbl.nw.userdata = panel;
+
+    lbl.nw.skin_element_name = "TimeLabel";
+    lbl.label    = "00:00:00";
+    lbl.callback = _panel_change_time_label;
+    panel->runtime_label = xitk_label_create (&lbl, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, panel->runtime_label, XITK_WIDGET_STATE_KEEP);
     xitk_set_widget_tips (panel->runtime_label, _("Total time: --:--:--"));
 
-    lbl.label             = "";
+    lbl.label = "";
 
-    lbl.skin_element_name = "TitleLabel";
-    lbl.callback          = _panel_change_display_mode;
-    panel->title_label =  xitk_label_create (panel->widget_list, panel->gui->skin_config, &lbl);
+    lbl.nw.skin_element_name = "TitleLabel";
+    lbl.callback = _panel_change_display_mode;
+    panel->title_label =  xitk_label_create (&lbl, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, panel->title_label, XITK_WIDGET_STATE_KEEP);
 
-    lbl.skin_element_name = "AudioLabel";
-    lbl.callback          = panel_audio_lang_list;
-    panel->audiochan_label = xitk_label_create (panel->widget_list, panel->gui->skin_config, &lbl);
+    lbl.nw.skin_element_name = "AudioLabel";
+    lbl.callback = panel_audio_lang_list;
+    panel->audiochan_label = xitk_label_create (&lbl, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, panel->audiochan_label, XITK_WIDGET_STATE_KEEP);
 
-    lbl.skin_element_name = "SpuLabel";
-    lbl.callback          = panel_spu_lang_list;
-    panel->spuid_label =  xitk_label_create (panel->widget_list, panel->gui->skin_config, &lbl);
+    lbl.nw.skin_element_name = "SpuLabel";
+    lbl.callback = panel_spu_lang_list;
+    panel->spuid_label =  xitk_label_create (&lbl, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, panel->spuid_label, XITK_WIDGET_STATE_KEEP);
   }
 
   {
     xitk_slider_widget_t sl;
-    XITK_WIDGET_INIT (&sl);
-    sl.motion_userdata   = panel;
-    sl.callback          = NULL;
-    sl.userdata          = NULL;
-    sl.min               = 0;
 
-    sl.skin_element_name = "SliderPlay";
-    sl.max               = 65535;
-    sl.step              = sl.max / 50;
-    sl.motion_callback   = panel_slider_play_cb;
-    panel->w[_W_slider_play] = xitk_slider_create (panel->widget_list, panel->gui->skin_config, &sl);
+    sl.nw.wl       = panel->widget_list;
+    sl.nw.userdata = panel;
+    sl.callback    = NULL;
+    sl.min         = 0;
+
+    sl.nw.skin_element_name = "SliderPlay";
+    sl.max             = 65535;
+    sl.step            = sl.max / 50;
+    sl.motion_callback = panel_slider_play_cb;
+    panel->w[_W_slider_play] = xitk_slider_create (&sl, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, panel->w[_W_slider_play],
       (panel->gui->playlist.num ? XITK_WIDGET_STATE_ENABLE : 0) | XITK_WIDGET_STATE_VISIBLE);
     xitk_widget_mode (panel->w[_W_slider_play], WIDGET_KEEP_FOCUS, 0);
     xitk_set_widget_tips (panel->w[_W_slider_play], _("Stream playback position slider"));
     xitk_slider_reset (panel->w[_W_slider_play]);
 
-    sl.skin_element_name = "SliderVol";
-    sl.max               = (panel->gui->mixer.method == SOUND_CARD_MIXER) ? 100 : 200;
-    sl.step              = sl.max / 20;
-    sl.motion_callback   = panel_slider_mixer_cb;
-    panel->w[_W_slider_mixer] = xitk_slider_create (panel->widget_list, panel->gui->skin_config, &sl);
+    sl.nw.skin_element_name = "SliderVol";
+    sl.max             = (panel->gui->mixer.method == SOUND_CARD_MIXER) ? 100 : 200;
+    sl.step            = sl.max / 20;
+    sl.motion_callback = panel_slider_mixer_cb;
+    panel->w[_W_slider_mixer] = xitk_slider_create (&sl, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, panel->w[_W_slider_mixer], XITK_WIDGET_STATE_VISIBLE);
     xitk_slider_reset (panel->w[_W_slider_mixer]);
   }
 
   {
     xitk_button_widget_t b;
-    XITK_WIDGET_INIT (&b);
 
-    b.callback          = NULL;
+    b.nw.wl    = panel->widget_list;
+    b.callback = NULL;
 
-    b.skin_element_name = "Pause";
-    b.state_callback    = gui_pause;
-    b.userdata          = panel->gui;
-    panel->w[_W_pause] = xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "Pause";
+    b.nw.userdata          = panel->gui;
+    b.state_callback = gui_pause;
+    panel->w[_W_pause] = xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, panel->w[_W_pause],
       XITK_WIDGET_STATE_ENABLE | XITK_WIDGET_STATE_VISIBLE | XITK_WIDGET_STATE_IMMEDIATE);
     xitk_set_widget_tips (panel->w[_W_pause], _("Pause/Resume playback"));
 
-    b.skin_element_name = "Mute";
-    b.state_callback    = panel_toggle_audio_mute;
-    b.userdata          = panel;
-    panel->w[_W_mute] = xitk_button_create (panel->widget_list, panel->gui->skin_config, &b);
+    b.nw.skin_element_name = "Mute";
+    b.nw.userdata          = panel;
+    b.state_callback = panel_toggle_audio_mute;
+    panel->w[_W_mute] = xitk_button_create (&b, panel->gui->skin_config);
     xitk_add_widget (panel->widget_list, panel->w[_W_mute], XITK_WIDGET_STATE_VISIBLE);
   }
 
