@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2021 the xine project
+ * Copyright (C) 2000-2022 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -148,7 +148,7 @@ static slx_entry_t **skins_get_slx_entries(gGui_t *gui, char *url) {
                       slx.skin.version = atoi(skin_prop->value);
                     }
                     else if(!strcasecmp(skin_prop->name, "MAINTAINED")) {
-                      slx.skin.maintained = get_bool_value(skin_prop->value);
+                      slx.skin.maintained = xitk_get_bool_value (skin_prop->value);
                     }
                   }
                 }
@@ -366,7 +366,7 @@ static void download_skin_select (xitk_widget_t *w, void *data, int state) {
   do {
     char skindir[2048], *skinname, *skinend, *end = skindir + sizeof (skindir) - 8;
     char *filename;
-    struct stat  st;
+    int err;
 
     if (!network_download (skd->slxs[selected]->skin.href, &download)) {
       gui_msg (gui, XUI_MSG_ERROR, _("Unable to download '%s': %s"), skd->slxs[selected]->skin.href, download.error);
@@ -390,12 +390,12 @@ static void download_skin_select (xitk_widget_t *w, void *data, int state) {
     if (skinname >= end)
       skinname = end;
     /* Make sure to have the directory */
-    if (stat (skindir, &st) < 0) {
-      (void)mkdir_safe (skindir);
-      if (stat (skindir, &st) < 0) {
-        gui_msg (gui, XUI_MSG_ERROR, _("Unable to create '%s' directory: %s."), skindir, strerror(errno));
-        break;
-      }
+    err = mkdir_safe (skindir);
+    if (!err) {
+      /* xprintf (gui->xine, XINE_VERBOSITY_DEBUG, "skins_download: created dir %s.\n", skindir) */;
+    } else if (err != EEXIST) {
+      gui_msg (gui, XUI_MSG_ERROR, _("Unable to create '%s' directory: %s."), skindir, strerror (err));
+      break;
     }
 
     {
@@ -543,8 +543,9 @@ void skin_download (gGui_t *gui, char *url) {
     x = 15;
     y = 34 + PREVIEW_HEIGHT + 14;
 
-    XITK_WIDGET_INIT (&br);
-    XITK_WIDGET_INIT (&lb);
+    br.nw.wl                   = skd->widget_list;
+    br.nw.skin_element_name    = NULL;
+    br.nw.userdata             = skd;
 
     br.arrow_up.skin_element_name    = NULL;
     br.slider.skin_element_name      = NULL;
@@ -555,8 +556,7 @@ void skin_download (gGui_t *gui, char *url) {
     br.browser.entries               = (const char *const *)skd->entries;
     br.callback                      = download_skin_preview;
     br.dbl_click_callback            = NULL;
-    br.userdata                      = skd;
-    skd->browser = xitk_noskin_browser_create (skd->widget_list, &br,
+    skd->browser = xitk_noskin_browser_create (&br,
       x + 5, y + 5, WINDOW_WIDTH - (30 + 10 + 16), 20, 16, br_fontname);
     xitk_add_widget (skd->widget_list, skd->browser, XITK_WIDGET_STATE_ENABLE | XITK_WIDGET_STATE_VISIBLE);
 
@@ -570,15 +570,17 @@ void skin_download (gGui_t *gui, char *url) {
     y = WINDOW_HEIGHT - (23 + 15);
     x = 15;
 
+    lb.nw.wl                   = skd->widget_list;
+    lb.nw.skin_element_name    = NULL;
+    lb.nw.userdata             = skd;
+
     lb.button_type       = CLICK_BUTTON;
     lb.label             = _("Load");
     lb.align             = ALIGN_CENTER;
     lb.callback          = download_skin_select;
     lb.state_callback    = NULL;
-    lb.userdata          = skd;
-    lb.skin_element_name = NULL;
-    widget =  xitk_noskin_labelbutton_create (skd->widget_list,
-      &lb, x, y, 100, 23, XITK_NOSKIN_TEXT_NORM, XITK_NOSKIN_TEXT_NORM, XITK_NOSKIN_TEXT_INV, btnfontname);
+    widget =  xitk_noskin_labelbutton_create (&lb,
+      x, y, 100, 23, XITK_NOSKIN_TEXT_NORM, XITK_NOSKIN_TEXT_NORM, XITK_NOSKIN_TEXT_INV, btnfontname);
     xitk_add_widget (skd->widget_list, widget, XITK_WIDGET_STATE_ENABLE | XITK_WIDGET_STATE_VISIBLE);
 
     x = WINDOW_WIDTH - (100 + 15);
@@ -588,10 +590,8 @@ void skin_download (gGui_t *gui, char *url) {
     lb.align             = ALIGN_CENTER;
     lb.callback          = download_skin_exit;
     lb.state_callback    = NULL;
-    lb.userdata          = skd;
-    lb.skin_element_name = NULL;
-    widget =  xitk_noskin_labelbutton_create (skd->widget_list,
-      &lb, x, y, 100, 23, XITK_NOSKIN_TEXT_NORM, XITK_NOSKIN_TEXT_NORM, XITK_NOSKIN_TEXT_INV, btnfontname);
+    widget =  xitk_noskin_labelbutton_create (&lb,
+      x, y, 100, 23, XITK_NOSKIN_TEXT_NORM, XITK_NOSKIN_TEXT_NORM, XITK_NOSKIN_TEXT_INV, btnfontname);
     xitk_add_widget (skd->widget_list, widget, XITK_WIDGET_STATE_ENABLE | XITK_WIDGET_STATE_VISIBLE);
 
     skd->widget_key = xitk_be_register_event_handler ("skdloader", skd->xwin, download_skin_event, skd, NULL, NULL);
