@@ -1,18 +1,18 @@
 /*
- * Copyright (C) 2000-2013 the xine project
+ * Copyright (C) 2000-2021 the xine project
  *
  * This file is part of xine, a unix video player.
- * 
+ *
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * xine is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
@@ -41,7 +41,7 @@
  *  James Courtier-Dutton <James@superbug.demon.co.uk> for educating the author
  *    in the details of YUV style data formats.
  *
- *  Thomas streich 
+ *  Thomas Östreich
  *    for the yuy2toyv12() function (see below)
  *
  */
@@ -63,15 +63,18 @@
 #include <png.h>
 
 #include "common.h"
+#include "snapshot.h"
 
 #define PIXSZ 3
 #define BIT_DEPTH 8
 
-static snapshot_messenger_t error_msg_cb;
-static snapshot_messenger_t info_msg_cb;
-static void *msg_cb_data;
+typedef struct {
+  snapshot_messenger_t  error_msg_cb;
+  snapshot_messenger_t  info_msg_cb;
+  void                 *msg_cb_data;
+} msg_cb_t;
 
-static char *snap_build_filename(const char *mrl) {
+static char *snap_build_filename (gGui_t *gui, const char *mrl) {
   char         *buffer;
   char          basename[XITK_NAME_MAX + 1] = "xine_snapshot";
   char         *p = strrchr(mrl, '/');
@@ -80,23 +83,23 @@ static char *snap_build_filename(const char *mrl) {
 
   if(p && (strlen(p) > 1)) {
     char *ext;
-    
+
     p++;
-    
+
     strlcpy(basename, p, sizeof(basename));
-    
+
     if((ext = strrchr(basename, '.')) != NULL)
       *ext = '\0';
-    
+
   }
 
   for(i = 1;; i++) {
-    asprintf(&buffer, "%s/%s-%d%s", gGui->snapshot_location, basename, i, ".png");
-    if(((stat(buffer, &sstat)) == -1) && (errno == ENOENT))
+    buffer = xitk_asprintf ("%s/%s-%d%s", gui->snapshot_location, basename, i, ".png");
+    if(!buffer || ((stat(buffer, &sstat) == -1) && (errno == ENOENT)))
       break;
     free(buffer);
   }
-  
+
   return buffer;
 }
 
@@ -106,24 +109,31 @@ static char *snap_build_filename(const char *mrl) {
 
 static void user_error_fn(png_structp png_ptr, png_const_charp error_msg)
 {
+  msg_cb_t *cbs = png_get_error_ptr(png_ptr);
 
-  if(error_msg_cb) {
+  if(cbs && cbs->error_msg_cb) {
     char *uerror;
 
-    asprintf(&uerror, "%s%s\n", _("Error: "), error_msg);
-    error_msg_cb(msg_cb_data, uerror);
-    free(uerror);
+    uerror = xitk_asprintf("%s%s\n", _("Error: "), error_msg);
+    if (uerror) {
+      cbs->error_msg_cb(cbs->msg_cb_data, uerror);
+      free(uerror);
+    }
   }
 }
 
 static void user_warning_fn(png_structp png_ptr, png_const_charp warning_msg)
 {
-  if(error_msg_cb) {
+  msg_cb_t *cbs = png_get_error_ptr(png_ptr);
+
+  if(cbs && cbs->error_msg_cb) {
     char *uerror;
 
-    asprintf(&uerror, "%s%s\n", _("Error: "), warning_msg);
-    error_msg_cb(msg_cb_data, uerror);
-    free(uerror);
+    uerror = xitk_asprintf("%s%s\n", _("Error: "), warning_msg);
+    if (uerror) {
+      cbs->error_msg_cb(cbs->msg_cb_data, uerror);
+      free(uerror);
+    }
   }
 }
 
@@ -132,6 +142,9 @@ static void user_warning_fn(png_structp png_ptr, png_const_charp warning_msg)
  */
 static void write_row_callback( png_structp png_ptr, png_uint_32 row, int pass)
 {
+  (void)png_ptr;
+  (void)row;
+  (void) pass;
 }
 
 
@@ -139,8 +152,43 @@ static void write_row_callback( png_structp png_ptr, png_uint_32 row, int pass)
 
 #include <xine.h>
 
-void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
-  snapshot_messenger_t info_mcb, void *mcb_data) {
+static int _encode(png_structp struct_ptr, png_infop info_ptr, png_bytep *rows, int width, int height,
+                   FILE *fp, snapshot_messenger_t error_mcb, void *mcb_data ) {
+
+#ifdef PNG_SETJMP_SUPPORTED /* libpng 1.0.5 has no png_jmpbuf */
+    /* Set up long jump callback for PNG parser */
+#if defined(PNG_INFO_IMAGE_SUPPORTED)
+    if (setjmp (png_jmpbuf (struct_ptr)))
+#else
+    if (setjmp (struct_ptr->jmpbuf))
+#endif
+    {
+      if (error_mcb)
+        error_mcb (mcb_data, _("PNG Parsing failed\n"));
+      return -1;
+    }
+#endif
+
+    png_set_filter (struct_ptr, 0, PNG_FILTER_NONE | PNG_FILTER_VALUE_NONE);
+    png_init_io (struct_ptr, fp);
+    png_set_write_status_fn (struct_ptr, write_row_callback);
+    png_set_IHDR (struct_ptr, info_ptr, width, height,
+      BIT_DEPTH, PNG_COLOR_TYPE_RGB, PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
+
+#if defined(PNG_INFO_IMAGE_SUPPORTED)
+    png_set_rows (struct_ptr, info_ptr, rows);
+    png_write_png (struct_ptr, info_ptr, PNG_TRANSFORM_IDENTITY, NULL);
+#else
+    png_write_info (struct_ptr, info_ptr);
+    png_write_image (struct_ptr, rows);
+    png_write_end (struct_ptr, NULL);
+#endif
+
+    return 0;
+}
+
+void create_snapshot (gGui_t *gui, const char *mrl,
+  snapshot_messenger_t error_mcb, snapshot_messenger_t info_mcb, void *mcb_data) {
 
   xine_grab_video_frame_t *xgvf = NULL;
 
@@ -153,15 +201,13 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
 
   int i;
 
-  error_msg_cb = error_mcb;
-  info_msg_cb  = info_mcb;
-  msg_cb_data  = mcb_data;
+  msg_cb_t cbs = { error_mcb, info_mcb, mcb_data };
 
   do {
-    xgvf = xine_new_grab_video_frame (gGui->stream);
+    xgvf = xine_new_grab_video_frame (gui->stream);
     if (!xgvf) {
-      if (error_msg_cb)
-        error_msg_cb (msg_cb_data, _("xine_grab_video_frame.grab () failed\n"));
+      if (error_mcb)
+        error_mcb (mcb_data, _("xine_grab_video_frame.grab () failed\n"));
       break;
     }
 
@@ -176,49 +222,53 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
     xgvf->flags  = 0;
 
     if (xgvf->grab (xgvf) != 0) {
-      if (error_msg_cb)
-        error_msg_cb (msg_cb_data, _("xine_new_grab_video_frame () failed\n"));
+      if (error_mcb)
+        error_mcb (mcb_data, _("xine_new_grab_video_frame () failed\n"));
       break;
     }
 
     if ((!xgvf->width) || (!xgvf->height)) {
-      if(error_msg_cb) {
+      if(error_mcb) {
         char *umessage;
 
-        asprintf (&umessage, "%s%dx%d%s\n",
+        umessage = xitk_asprintf ("%s%dx%d%s\n",
           _("Wrong image size: "), xgvf->width, xgvf->height, _(". Snapshot aborted."));
-        error_msg_cb (msg_cb_data, umessage);
-        free (umessage);
+        if (umessage) {
+          error_mcb (mcb_data, umessage);
+          free (umessage);
+        }
       }
       break;
     }
 
     /* open file */
-    file_name = snap_build_filename (mrl);
-    if ((fp = fopen (file_name, "wb")) == NULL) {
-      if (error_msg_cb) {
+    file_name = snap_build_filename (gui, mrl);
+    if (!file_name || (fp = fopen (file_name, "wb")) == NULL) {
+      if (error_mcb) {
         char *umessage;
 
-        asprintf (&umessage, "%s (%s)\n", _("File open failed"), file_name);
-        error_msg_cb (msg_cb_data, umessage);
-        free (umessage);
+        umessage = xitk_asprintf ("%s (%s)\n", _("File open failed"), file_name);
+        if (umessage) {
+          error_mcb (mcb_data, umessage);
+          free (umessage);
+        }
       }
       break;
     }
 
     /* get write struct */
-    struct_ptr = png_create_write_struct (PNG_LIBPNG_VER_STRING, (png_voidp)0, user_error_fn, user_warning_fn);
+    struct_ptr = png_create_write_struct (PNG_LIBPNG_VER_STRING, &cbs, user_error_fn, user_warning_fn);
     if (!struct_ptr) {
-      if (error_msg_cb)
-        error_msg_cb (msg_cb_data, _("png_create_write_struct() failed\n"));
+      if (error_mcb)
+        error_mcb (mcb_data, _("png_create_write_struct() failed\n"));
       break;
     }
 
     /* get info struct */
     info_ptr = png_create_info_struct (struct_ptr);
     if (!info_ptr) {
-      if (error_msg_cb)
-        error_msg_cb (msg_cb_data, _("png_create_info_struct() failed\n"));
+      if (error_mcb)
+        error_mcb (mcb_data, _("png_create_info_struct() failed\n"));
       break;
     }
 
@@ -229,40 +279,16 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
     for (i = 0; i < xgvf->height; i++)
       rows[i] = xgvf->img + i * xgvf->width * PIXSZ;
 
-#ifdef PNG_SETJMP_SUPPORTED /* libpng 1.0.5 has no png_jmpbuf */
-    /* Set up long jump callback for PNG parser */
-#if defined(PNG_INFO_IMAGE_SUPPORTED)
-    if (setjmp (png_jmpbuf (struct_ptr)))
-#else
-    if (setjmp (struct_ptr->jmpbuf))
-#endif
-    {
-      if (error_msg_cb)
-        error_msg_cb (msg_cb_data, _("PNG Parsing failed\n"));
+    if (_encode(struct_ptr, info_ptr, rows, xgvf->width, xgvf->height, fp, error_mcb, mcb_data) < 0)
       break;
-    }
-#endif
 
-    png_set_filter (struct_ptr, 0, PNG_FILTER_NONE | PNG_FILTER_VALUE_NONE);
-    png_init_io (struct_ptr, fp);
-    png_set_write_status_fn (struct_ptr, write_row_callback);
-    png_set_IHDR (struct_ptr, info_ptr, xgvf->width, xgvf->height,
-      BIT_DEPTH, PNG_COLOR_TYPE_RGB, PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
-
-#if defined(PNG_INFO_IMAGE_SUPPORTED)
-    png_set_rows (struct_ptr, info_ptr, rows);
-    png_write_png (struct_ptr, info_ptr, PNG_TRANSFORM_IDENTITY, NULL);
-#else
-    png_write_info (struct_ptr, info_ptr);
-    png_write_image (struct_ptr, rows);
-    png_write_end (struct_ptr, NULL);
-#endif
-
-    if (info_msg_cb) {
+    if (info_mcb) {
       char *umessage;
-      asprintf (&umessage, "%s%s\n", _("File written: "), file_name);
-      info_msg_cb (msg_cb_data, umessage);
-      free (umessage);
+      umessage = xitk_asprintf ("%s%s\n", _("File written: "), file_name);
+      if (umessage) {
+        info_mcb (mcb_data, umessage);
+        free (umessage);
+      }
     }
 
   } while (0);
@@ -317,10 +343,10 @@ struct prvt_image_s {
  *  This function was pinched from filter_yuy2tov12.c, part of
  *  transcode, a linux video stream processing tool
  *
- *  Copyright (C) Thomas streich - June 2001
+ *  Copyright (C) Thomas Östreich - June 2001
  *
  *  Thanks Thomas
- *      
+ *
  */
 static void yuy2toyv12( struct prvt_image_s *image )
 {
@@ -333,7 +359,7 @@ static void yuy2toyv12( struct prvt_image_s *image )
     uint8_t *v = image->v;
 
     uint8_t *input = image->yuy2;
-    
+
     int width  = image->width;
     int height = image->height;
 
@@ -341,16 +367,16 @@ static void yuy2toyv12( struct prvt_image_s *image )
 
     for (i=0; i<height; i+=2) {
       for (j=0; j<w2; j++) {
-	
+
 	/* packed YUV 422 is: Y[i] U[i] Y[i+1] V[i] */
 	*(y++) = *(input++);
 	*(u++) = *(input++);
 	*(y++) = *(input++);
 	*(v++) = *(input++);
       }
-      
+
       /* down sampling */
-      
+
       for (j=0; j<w2; j++) {
 	/* skip every second line for U and V */
 	*(y++) = *(input++);
@@ -793,7 +819,7 @@ static int scale_image( struct prvt_image_s *image )
   image->u_width = nu_width;
   image->v_width = nv_width;
 
-#ifdef DEBUG 
+#ifdef DEBUG
   fprintf(stderr, "  Post scaled\n    Width %d %d %d\n    Height %d %d %d\n",
     image->width, image->u_width, image->v_width,
     image->height, image->u_height, image->v_height );
@@ -884,10 +910,10 @@ static void rgb_free( struct prvt_image_s *image )
 static int prvt_image_alloc( struct prvt_image_s **image, int imgsize )
 {
   *image = (struct prvt_image_s*) calloc(1, sizeof( struct prvt_image_s ) );
-  
-  if (*image == NULL) 
+
+  if (*image == NULL)
     return 0;
-    
+
   (*image)->img = malloc( imgsize );
   if ((*image)->img == NULL) {
     free(*image);
@@ -895,7 +921,7 @@ static int prvt_image_alloc( struct prvt_image_s **image, int imgsize )
   }
   (*image)->scale_image_y = (*image)->scale_image_u = (*image)->scale_image_v = NULL;
   (*image)->yuy2_fudge_y = (*image)->yuy2_fudge_u = (*image)->yuy2_fudge_v = NULL;
-  
+
   return( 1 );
 }
 
@@ -906,7 +932,7 @@ static void prvt_image_free( struct prvt_image_s **image )
   free(image_p->file_name);
 
   free(image_p->img);
-   
+
   rgb_free ( image_p );
 
   png_free( image_p->struct_ptr, image_p->scale_image_y );
@@ -964,7 +990,7 @@ static void yv12_2_rgb( struct prvt_image_s *image )
 
 /***************************************************
  *
- *  Colour conversion from 
+ *  Colour conversion from
  *    http://www.inforamp.net/~poynton/notes/colour_and_gamma/ColorFAQ.html#RTFToC30
  *
  *  Thanks to Billy Biggs <vektor@dumbterm.net>
@@ -1012,7 +1038,8 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
   struct prvt_image_s *image;
   int width, height, ratio_code, format;
   double aspect, destaspect, scale;
-  
+  msg_cb_t cbs = { error_mcb, info_mcb, mcb_data };
+
 #ifdef DEBUG
   static int	   prof_scale_image = -1;
   static int	   prof_yuv2rgb     = -1;
@@ -1026,51 +1053,49 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
     prof_png = xine_profiler_allocate_slot ("snapshot convert to png");
 #endif /* DEBUG */
 
-  error_msg_cb = error_mcb;
-  info_msg_cb = info_mcb;
-  msg_cb_data = mcb_data;
+  err = xine_get_current_frame (gui->stream, &width, &height, &ratio_code, &format, NULL);
 
-  err = xine_get_current_frame(gGui->stream, &width, &height, &ratio_code, &format, NULL);
-  
   if (err == 0) {
-    error_msg_cb(msg_cb_data, _("xine_get_current_frame() failed\n"));
+    error_mcb(mcb_data, _("xine_get_current_frame() failed\n"));
     return;
   }
 
   if((!width) || (!height)) {
-    if(error_msg_cb) {
+    if(error_mcb) {
       char *umessage;
-      
-      asprintf(&umessage, "%s%dx%d%s\n", _("Wrong image size: "), width, height, _(". Snapshot aborted."));
-      error_msg_cb(msg_cb_data, umessage);
-      free(umessage);
+
+      umessage = xitk_asprintf("%s%dx%d%s\n", _("Wrong image size: "), width, height, _(". Snapshot aborted."));
+      if (umessage) {
+        error_mcb(mcb_data, umessage);
+        free(umessage);
+      }
     }
     return;
   }
-  
+
 #ifdef DEBUG
   printf("%s:allocating space for a %d x %d image\n", __FILE__, width, height);
 #endif
-  
+
   if ( ! prvt_image_alloc( &image, width*height*2 ) )
   {
-    error_msg_cb(msg_cb_data, _("prvt_image_alloc failed\n"));
+    error_mcb(mcb_data, _("prvt_image_alloc failed\n"));
     return;
   }
 
-  err = xine_get_current_frame(gGui->stream,
+  err = xine_get_current_frame (gui->stream,
 			       &image->width, &image->height, &image->ratio_code,
 			       &image->format, image->img);
-  
+
   if (err == 0) {
-    error_msg_cb(msg_cb_data, _("Framegrabber failed\n"));
+    error_mcb(mcb_data, _("Framegrabber failed\n"));
     prvt_image_free( &image );
     return;
   }
 
   /* the dxr3 driver does not allocate yuv buffers */
   if (! image->img) { /* image->u and image->v are always 0 for YUY2 */
-    error_msg_cb(msg_cb_data, _("Not supported for this video out driver\n"));
+    error_mcb(mcb_data, _("Not supported for this video out driver\n"));
     prvt_image_free( &image );
     return;
   }
@@ -1089,20 +1114,20 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
     case XINE_VO_ASPECT_4_3:
       destaspect = 4 / 3.0;
       break;
-    case XINE_VO_ASPECT_ANAMORPHIC: 
+    case XINE_VO_ASPECT_ANAMORPHIC:
       destaspect = 16 / 9.0;
       break;
-    case XINE_VO_ASPECT_DVB:      
+    case XINE_VO_ASPECT_DVB:
       destaspect = 2.11;
       break;
 #ifdef XINE_VO_ASPECT_DONT_TOUCH
     case XINE_VO_ASPECT_DONT_TOUCH:
-#endif 
-    default: 
+#endif
+    default:
       destaspect = 0;
       scale = 1;
 #ifdef DEBUG
-      printf( "Warning: unknown aspect ratio. will assume 1:1\n" ); 
+      printf( "Warning: unknown aspect ratio. will assume 1:1\n" );
 #endif
   }
 
@@ -1114,7 +1139,7 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
   fprintf(stderr, "  input aspect: %g  output aspect: %g  pixel aspect: %g ",
 	 aspect, destaspect, scale);
 #endif
-      
+
   if (scale > 0.99 && scale < 1.01) {
       image->scale_factor = 0x8000;
       image->scale_line = scale_line_1_1;
@@ -1137,7 +1162,7 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
   switch ( image->format ) {
     case XINE_IMGFMT_YV12:
 #ifdef DEBUG
-      printf( "XINE_IMGFMT_YV12\n" ); 
+      printf( "XINE_IMGFMT_YV12\n" );
 #endif
       image->y        = image->img;
       image->u        = image->img + (image->width*image->height);
@@ -1148,7 +1173,7 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
       image->v_height = ((image->height+1)/2);
       break;
 
-    case XINE_IMGFMT_YUY2: 
+    case XINE_IMGFMT_YUY2:
 #ifdef DEBUG
       printf( "XINE_IMGFMT_YUY2\n" );
 #endif
@@ -1159,9 +1184,9 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
       image->v_height = ((image->height+1)/2);
       break;
 
-    default:                
+    default:
 #ifdef DEBUG
-      printf( "Unknown\nError: Format Code %d Unknown\n", image->format ); 
+      printf( "Unknown\nError: Format Code %d Unknown\n", image->format );
       printf( "  ** Please report this error to andrew@anvil.org **\n" );
 #endif
       prvt_image_free( &image );
@@ -1170,16 +1195,18 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
 
   /**/
 
-  image->file_name = snap_build_filename(mrl);
-  
-  if ( (image->fp = fopen(image->file_name, "wb")) == NULL ) {
-    
-    if(error_msg_cb) {
+  image->file_name = snap_build_filename (gui, mrl);
+
+  if ( !image->file_name || (image->fp = fopen(image->file_name, "wb")) == NULL ) {
+
+    if(error_mcb) {
       char *umessage;
-      
-      asprintf(&umessage, "%s (%s)\n", _("File open failed"), image->file_name);
-      error_msg_cb(msg_cb_data, umessage);
-      free(umessage);
+
+      umessage = xitk_asprintf("%s (%s)\n", _("File open failed"), image->file_name);
+      if (umessage) {
+        error_mcb(mcb_data, umessage);
+        free(umessage);
+      }
     }
     prvt_image_free( &image );
     return;
@@ -1193,12 +1220,12 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
 
   image->struct_ptr = png_create_write_struct (
     PNG_LIBPNG_VER_STRING,
-    (png_voidp)0,
+    &cbs,
     user_error_fn,
     user_warning_fn);
 
   if (image->struct_ptr == NULL) {
-    error_msg_cb(msg_cb_data, _("png_create_write_struct() failed\n"));
+    error_mcb(mcb_data, _("png_create_write_struct() failed\n"));
     prvt_image_free( &image );
     return;
   }
@@ -1212,7 +1239,7 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
   image->info_ptr = png_create_info_struct(image->struct_ptr);
 
   if (image->info_ptr == NULL) {
-    error_msg_cb(msg_cb_data, _("png_create_info_struct() failed\n"));
+    error_mcb(mcb_data, _("png_create_info_struct() failed\n"));
     prvt_image_free( &image );
     return;
   }
@@ -1232,7 +1259,7 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
   if (setjmp(image->struct_ptr->jmpbuf))
 #endif
   {
-    error_msg_cb(msg_cb_data, _("PNG Parsing failed\n"));
+    error_mcb(mcb_data, _("PNG Parsing failed\n"));
     prvt_image_free( &image );
     return;
   }
@@ -1246,7 +1273,7 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
     printf("  Convert YUY2 to YV12\n" );
 #endif
     if ( yuy2_fudge( image ) == 0 ) {
-      error_msg_cb(msg_cb_data, _("Error: yuy2_fudge failed\n"));
+      error_mcb(mcb_data, _("Error: yuy2_fudge failed\n"));
       prvt_image_free( &image );
       return;
     }
@@ -1278,7 +1305,7 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
 
   if ( !rgb_alloc( image ) )
   {
-    error_msg_cb(msg_cb_data, _("rgb_alloc() failed\n"));
+    error_mcb(mcb_data, _("rgb_alloc() failed\n"));
     prvt_image_free( &image );
     return;
   }
@@ -1316,7 +1343,7 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
   printf("  png_init_io\n" );
 #endif
   png_init_io(image->struct_ptr, image->fp);
-  
+
 #ifdef DEBUG
   printf("  png_set_write_status_fn\n" );
 #endif
@@ -1327,15 +1354,15 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
 #ifdef DEBUG
   printf("  png_set_IHDR\n" );
 #endif
-  png_set_IHDR( 
-    image->struct_ptr, 
-    image->info_ptr, 
-    image->width, 
+  png_set_IHDR(
+    image->struct_ptr,
+    image->info_ptr,
+    image->width,
     image->height,
-    BIT_DEPTH, 
-    PNG_COLOR_TYPE_RGB, 
+    BIT_DEPTH,
+    PNG_COLOR_TYPE_RGB,
     PNG_INTERLACE_NONE,
-    PNG_COMPRESSION_TYPE_DEFAULT, 
+    PNG_COMPRESSION_TYPE_DEFAULT,
     PNG_FILTER_TYPE_DEFAULT);
 
   /**/
@@ -1356,11 +1383,13 @@ void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
 #endif
 
   /**/
-  if(info_msg_cb) {
+  if(info_mcb) {
     char *umessage;
-    asprintf(&umessage, "%s%s\n", _("File written: "), image->file_name);
-    info_msg_cb(msg_cb_data, umessage);
-    free(umessage);
+    umessage = xitk_asprintf("%s%s\n", _("File written: "), image->file_name);
+    if (umessage) {
+      info_mcb(mcb_data, umessage);
+      free(umessage);
+    }
   }
 
 #ifdef DEBUG
