@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2021 the xine project
+ * Copyright (C) 2000-2022 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -140,8 +140,6 @@ static int _button_input (_button_private_t *wp, const widget_event_t *event) {
   if (event->type == WIDGET_EVENT_KEY) {
     static const char k[] = {
       XITK_CTRL_KEY_PREFIX, XITK_KEY_RETURN,
-      XITK_CTRL_KEY_PREFIX, XITK_KEY_NUMPAD_ENTER,
-      XITK_CTRL_KEY_PREFIX, XITK_KEY_ISO_ENTER,
       ' ', 0
     };
     int i, n = sizeof (k) / sizeof (k[0]);
@@ -228,7 +226,7 @@ static xitk_widget_t *_xitk_button_create (_button_private_t *wp, const xitk_but
 
   wp->callback          = b->callback;
   wp->state_callback    = b->state_callback;
-  wp->userdata          = b->userdata;
+  wp->userdata          = b->nw.userdata;
 
   wp->w.width           = wp->skin.width / (wp->s5 ? 6 : 3);
   wp->w.height          = wp->skin.height;
@@ -244,17 +242,17 @@ static xitk_widget_t *_xitk_button_create (_button_private_t *wp, const xitk_but
 /*
  *
  */
-xitk_widget_t *xitk_button_create (xitk_widget_list_t *wl,
-  xitk_skin_config_t *skonfig, const xitk_button_widget_t *b) {
+xitk_widget_t *xitk_button_create (const xitk_button_widget_t *b, xitk_skin_config_t *skonfig) {
   _button_private_t *wp;
 
-  XITK_CHECK_CONSTITENCY(b);
-  wp = (_button_private_t *)xitk_widget_new (wl, sizeof (*wp));
+  _xitk_new_widget_check (&b->nw);
+
+  wp = (_button_private_t *)xitk_widget_new (b->nw.wl, sizeof (*wp));
   if (!wp)
     return NULL;
 
   wp->s5 = 0;  strlcpy (wp->skin_element_name,
-    b->skin_element_name && b->skin_element_name[0] ? b->skin_element_name : "-",
+    b->nw.skin_element_name && b->nw.skin_element_name[0] ? b->nw.skin_element_name : "-",
     sizeof (wp->skin_element_name));
   _button_read_skin (wp, skonfig);
 
@@ -264,8 +262,7 @@ xitk_widget_t *xitk_button_create (xitk_widget_list_t *wl,
 /*
  *
  */
-xitk_widget_t *xitk_noskin_button_create (xitk_widget_list_t *wl,
-  const xitk_button_widget_t *b, int x, int y, int width, int height) {
+xitk_widget_t *xitk_noskin_button_create (const xitk_button_widget_t *b, int x, int y, int width, int height) {
   static const char *noskin_names[] = {
     "XITK_NOSKIN_LEFT",
     "XITK_NOSKIN_RIGHT",
@@ -280,26 +277,26 @@ xitk_widget_t *xitk_noskin_button_create (xitk_widget_list_t *wl,
   unsigned int u = sizeof (noskin_names) / sizeof (noskin_names[0]);
   xitk_image_t *i;
 
-  XITK_CHECK_CONSTITENCY(b);
-  wp = (_button_private_t *)xitk_widget_new (wl, sizeof (*wp));
+  _xitk_new_widget_check (&b->nw);
+  wp = (_button_private_t *)xitk_widget_new (b->nw.wl, sizeof (*wp));
   if (!wp)
     return NULL;
 
   _b = *b;
-  if (_b.skin_element_name) {
+  if (_b.nw.skin_element_name) {
     for (u = 0; u < sizeof (noskin_names) / sizeof (noskin_names[0]); u++) {
-      if (!strcmp (_b.skin_element_name, noskin_names[u]))
+      if (!strcmp (_b.nw.skin_element_name, noskin_names[u]))
         break;
     }
-    _b.skin_element_name = NULL;
+    _b.nw.skin_element_name = NULL;
   }
   if (u == sizeof (noskin_names) / sizeof (noskin_names[0])) {
-    i = xitk_image_new (wl->xitk, NULL, 0, width * 3, height);
+    i = xitk_image_new (wp->w.wl->xitk, NULL, 0, width * 3, height);
     xitk_image_draw_bevel_three_state (i);
     wp->s5 = 0;
   } else {
     wp->s5 = 1;
-    if (xitk_shared_image (wl, noskin_names[u], width * 6, height, &i) == 1) {
+    if (xitk_shared_image (wp->w.wl, noskin_names[u], width * 6, height, &i) == 1) {
       i->last_state = XITK_IMG_STATE_DISABLED_SELECTED;
       if (u != 6)
         xitk_image_draw_bevel_three_state (i);
