@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2021 the xine project
+ * Copyright (C) 2000-2022 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -221,7 +221,9 @@ static void _combo_open (_combo_private_t *wp) {
   {
     xitk_browser_widget_t browser;
     /* Browser */
-    XITK_WIDGET_INIT (&browser);
+    browser.nw.wl                = wp->widget_list;
+    browser.nw.skin_element_name = NULL;
+    browser.nw.userdata          = wp;
     browser.arrow_up.skin_element_name    = NULL;
     browser.slider.skin_element_name      = NULL;
     browser.arrow_dn.skin_element_name    = NULL;
@@ -234,8 +236,7 @@ static void _combo_open (_combo_private_t *wp) {
     browser.dbl_click_callback            = NULL;
     browser.browser.max_displayed_entries = 5;
     browser.callback                      = _combo_select;
-    browser.userdata                      = (void *)wp;
-    wp->iw[_W_browser] = xitk_noskin_browser_create (wp->widget_list, &browser,
+    wp->iw[_W_browser] = xitk_noskin_browser_create (&browser,
       1, 1, itemw, itemh, -slidw, DEFAULT_FONT_10);
     xitk_dlist_add_tail (&wp->widget_list->list, &wp->iw[_W_browser]->node);
     wp->iw[_W_browser]->type |= WIDGET_GROUP_MEMBER | WIDGET_GROUP_COMBO;
@@ -541,12 +542,10 @@ void xitk_combo_update_list(xitk_widget_t *w, const char *const *const list, int
 /*
  *
  */
-static xitk_widget_t *_combo_create (xitk_widget_list_t *wl, xitk_combo_widget_t *c,
+static xitk_widget_t *_combo_create (const xitk_combo_widget_t *c,
   const char *skin_element_name, _combo_private_t *wp, int visible, int enable) {
   const char * const *entries = c->entries;
 
-  ABORT_IF_NULL(wl);
-
   wp->xwin = NULL;
 
   if (skin_element_name) {
@@ -557,10 +556,10 @@ static xitk_widget_t *_combo_create (xitk_widget_list_t *wl, xitk_combo_widget_t
     wp->skin_element_name[0] = 0;
   }
   wp->iw[_W_combo]      = &wp->w;
-  wp->parent_wlist      = wl;
+  wp->parent_wlist      = wp->w.wl;
   wp->parent_wkey       = c->parent_wkey;
   wp->callback          = c->callback;
-  wp->userdata          = c->userdata;
+  wp->userdata          = c->nw.userdata;
   wp->num_entries       = 0x7fffffff;
   wp->entries           = _combo_copy_string_list (c->entries, &wp->num_entries);
   wp->selected          = -1;
@@ -587,40 +586,36 @@ static xitk_widget_t *_combo_create (xitk_widget_list_t *wl, xitk_combo_widget_t
 /*
  *
  */
-xitk_widget_t *xitk_combo_create (xitk_widget_list_t *wl,
-  xitk_skin_config_t *skonfig, xitk_combo_widget_t *c) {
+xitk_widget_t *xitk_combo_create (const xitk_combo_widget_t *c, xitk_skin_config_t *skonfig) {
   _combo_private_t        *wp;
   xitk_button_widget_t     b;
   xitk_label_widget_t      lbl;
 
-  XITK_CHECK_CONSTITENCY(c);
+  _xitk_new_widget_check (&c->nw);
 
-  XITK_WIDGET_INIT (&b);
-  XITK_WIDGET_INIT(&lbl);
-
-  wp = (_combo_private_t *)xitk_widget_new (wl, sizeof (*wp));
+  wp = (_combo_private_t *)xitk_widget_new (c->nw.wl, sizeof (*wp));
   if (!wp)
     return NULL;
 
   /* Create label and button (skinable) */
-  lbl.label             = "";
-  lbl.skin_element_name = c->skin_element_name;
-  lbl.callback          = NULL;
-  lbl.userdata          = NULL;
-  if ((wp->iw[_W_label] = xitk_label_create (wl, skonfig, &lbl))) {
+  _xitk_new_widget_init (&lbl.nw, wp);
+  lbl.nw.skin_element_name = c->nw.skin_element_name;
+  lbl.label    = "";
+  lbl.callback = NULL;
+  if ((wp->iw[_W_label] = xitk_label_create (&lbl, skonfig))) {
     xitk_widget_set_parent (wp->iw[_W_label], &wp->w);
     wp->iw[_W_label]->type |= WIDGET_GROUP_MEMBER | WIDGET_GROUP_COMBO;
-    xitk_dlist_add_tail (&wl->list, &wp->iw[_W_label]->node);
+    xitk_dlist_add_tail (&wp->w.wl->list, &wp->iw[_W_label]->node);
   }
 
-  b.skin_element_name = c->skin_element_name;
-  b.callback          = NULL;
-  b.state_callback    = _combo_rollunroll;
-  b.userdata          = (void *)wp;
-  if ((wp->iw[_W_button] = xitk_button_create (wl, skonfig, &b))) {
+  _xitk_new_widget_init (&b.nw, wp);
+  b.nw.skin_element_name = c->nw.skin_element_name;
+  b.callback       = NULL;
+  b.state_callback = _combo_rollunroll;
+  if ((wp->iw[_W_button] = xitk_button_create (&b, skonfig))) {
     xitk_widget_set_parent (wp->iw[_W_button], &wp->w);
     wp->iw[_W_button]->type |= WIDGET_GROUP_MEMBER | WIDGET_GROUP_COMBO;
-    xitk_dlist_add_tail (&wl->list, &wp->iw[_W_button]->node);
+    xitk_dlist_add_tail (&wp->w.wl->list, &wp->iw[_W_button]->node);
   }
 
   {
@@ -636,8 +631,8 @@ xitk_widget_t *xitk_combo_create (xitk_widget_list_t *wl,
     (void) xitk_set_widget_pos(wp->iw[_W_button], x, y);
   }
   {
-    const xitk_skin_element_info_t *info = xitk_skin_get_info (skonfig, c->skin_element_name);
-    return _combo_create (wl, c, c->skin_element_name ? c->skin_element_name : "", wp,
+    const xitk_skin_element_info_t *info = xitk_skin_get_info (skonfig, c->nw.skin_element_name);
+    return _combo_create (c, c->nw.skin_element_name ? c->nw.skin_element_name : "", wp,
       info ? (info->visibility ? 1 : -1) : 0, info ? info->enability : 0);
   }
 }
@@ -645,54 +640,46 @@ xitk_widget_t *xitk_combo_create (xitk_widget_list_t *wl,
 /*
  *  ******************************************************************************
  */
-xitk_widget_t *xitk_noskin_combo_create (xitk_widget_list_t *wl,
-  xitk_combo_widget_t *c, int x, int y, int width) {
+xitk_widget_t *xitk_noskin_combo_create (const xitk_combo_widget_t *c, int x, int y, int width) {
   _combo_private_t     *wp;
   xitk_button_widget_t  b;
   xitk_label_widget_t   lbl;
 
-  ABORT_IF_NULL(wl);
+  _xitk_new_widget_check (&c->nw);
 
-  XITK_CHECK_CONSTITENCY(c);
-
-  wp = (_combo_private_t *)xitk_widget_new (wl, sizeof (*wp));
+  wp = (_combo_private_t *)xitk_widget_new (c->nw.wl, sizeof (*wp));
   if (!wp)
     return NULL;
 
-  XITK_WIDGET_INIT (&b);
-  XITK_WIDGET_INIT(&lbl);
-
-
   /* Create label and button (skinable) */
   {
     xitk_font_t    *fs;
     int             height;
 
-    fs = xitk_font_load_font (wl->xitk, DEFAULT_FONT_10);
+    fs = xitk_font_load_font (wp->w.wl->xitk, DEFAULT_FONT_10);
     height = xitk_font_get_string_height(fs, " ") + 6;
     xitk_font_unload_font(fs);
 
-    lbl.skin_element_name = "XITK_NOSKIN_INNER";
-    lbl.label             = "";
-    lbl.callback          = NULL;
-    lbl.userdata          = NULL;
-    if ((wp->iw[_W_label] = xitk_noskin_label_create (wl, &lbl,
+    _xitk_new_widget_init (&lbl.nw, wp);
+    lbl.nw.skin_element_name = "XITK_NOSKIN_INNER";
+    lbl.label    = "";
+    lbl.callback = NULL;
+    if ((wp->iw[_W_label] = xitk_noskin_label_create (&lbl,
       x, y, (width - height), height, DEFAULT_FONT_10))) {
       xitk_widget_set_parent (wp->iw[_W_label], &wp->w);
       wp->iw[_W_label]->type |= WIDGET_GROUP_MEMBER | WIDGET_GROUP_COMBO;
-      xitk_dlist_add_tail (&wl->list, &wp->iw[_W_label]->node);
+      xitk_dlist_add_tail (&wp->w.wl->list, &wp->iw[_W_label]->node);
     }
 
-    b.skin_element_name = "XITK_NOSKIN_DOWN";
-    b.callback          = NULL;
-    b.state_callback    = _combo_rollunroll;
-    b.userdata          = (void *)wp;
+    _xitk_new_widget_init (&b.nw, wp);
+    b.nw.skin_element_name = "XITK_NOSKIN_DOWN";
+    b.callback       = NULL;
+    b.state_callback = _combo_rollunroll;
 
-    if ((wp->iw[_W_button] = xitk_noskin_button_create (wl, &b,
-      x + (width - height), y, height, height))) {
+    if ((wp->iw[_W_button] = xitk_noskin_button_create (&b, x + (width - height), y, height, height))) {
       xitk_widget_set_parent (wp->iw[_W_button], &wp->w);
       wp->iw[_W_button]->type |= WIDGET_GROUP_MEMBER | WIDGET_GROUP_COMBO;
-      xitk_dlist_add_tail (&wl->list, &wp->iw[_W_button]->node);
+      xitk_dlist_add_tail (&wp->w.wl->list, &wp->iw[_W_button]->node);
     }
 
     wp->w.x = x;
@@ -701,6 +688,5 @@ xitk_widget_t *xitk_noskin_combo_create (xitk_widget_list_t *wl,
     wp->w.height = height;
   }
 
-  return _combo_create (wl, c, NULL, wp, 0, 0);
+  return _combo_create (c, NULL, wp, 0, 0);
 }
-
