@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2021 the xine project
+ * Copyright (C) 2000-2022 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -23,16 +23,13 @@
 #define HAVE_XITK_IMAGE_H
 
 typedef struct {
-  int                               magic;
-  const char                       *skin_element_name;
+  xitk_new_widget_t nw;
 } xitk_image_widget_t;
 
 /** Create an image widget type. */
-xitk_widget_t *xitk_image_create (xitk_widget_list_t *wl,
-  xitk_skin_config_t *skonfig, const xitk_image_widget_t *im);
+xitk_widget_t *xitk_image_create (const xitk_image_widget_t *iw, xitk_skin_config_t *skonfig);
 
 /** Same as above, without skin. */
-xitk_widget_t *xitk_noskin_image_create (xitk_widget_list_t *wl,
-  const xitk_image_widget_t *im, xitk_image_t *image, int x, int y);
+xitk_widget_t *xitk_noskin_image_create (const xitk_image_widget_t *iw, xitk_image_t *image, int x, int y);
 
 #endif
