@@ -22,23 +22,20 @@
 #define HAVE_XITK_INTBOX_H
 
 typedef struct {
-  int                    magic;
-
-  const char            *skin_element_name;
+  xitk_new_widget_t     nw;
 
   int                    min, max, step, value;
   enum {
-    INTBOX_FMT_DECIMAL,
-    INTBOX_FMT_0x,
-    INTBOX_FMT_HASH
+    INTBOX_FMT_DECIMAL,   /**<<       0,   12345,     -55787, ... */
+    INTBOX_FMT_0x,        /**<<     0x0,  0x3039, 0xffff2615, ... */
+    INTBOX_FMT_HASH,      /**<<      #0,   #3039,  #ffff2615, ... */
+    INTBOX_FMT_SECONDS,   /**<< 0:00:00, 3:25:45,  -15:29:47, ... */
+    INTBOX_FMT_MILLISECS  /**<<   0.000,  12.345,    -55.787, ... */
   }                      fmt;
-
   xitk_state_callback_t  callback;
-  void                  *userdata;
 } xitk_intbox_widget_t;
 
-xitk_widget_t *xitk_noskin_intbox_create(xitk_widget_list_t *wl,
-  xitk_intbox_widget_t *ib, int x, int y, int width, int height);
-void xitk_intbox_set_value(xitk_widget_t *, int);
-int xitk_intbox_get_value(xitk_widget_t *);
+xitk_widget_t *xitk_noskin_intbox_create (const xitk_intbox_widget_t *ib, int x, int y, int width, int height);
+int xitk_intbox_set_value (xitk_widget_t *iw, int value);
+int xitk_intbox_get_value (xitk_widget_t *iw);
 #endif
