@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2021 the xine project
+ * Copyright (C) 2000-2022 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -476,19 +476,34 @@ int xitk_label_change_label(xitk_widget_t *w, const char *newlabel) {
 /*
  *
  */
-static int _label_click (_label_private_t *wp, int button, int bUp, int x, int y) {
+static int _label_input (_label_private_t *wp, const widget_event_t *event) {
   int  ret = 0;
 
-  (void)x;
-  (void)y;
-  if (wp && ((wp->w.type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL)) {
-    if (button == 1) {
-      if (wp->callback) {
-        if (bUp)
-          wp->callback (&wp->w, wp->userdata);
-        ret = 1;
-      }
+  if (event->type == WIDGET_EVENT_KEY) {
+    static const char k[] = {
+      XITK_CTRL_KEY_PREFIX, XITK_KEY_RETURN,
+      ' ', 0
+    };
+    int i, n = sizeof (k) / sizeof (k[0]);
+
+    if (event->modifier & ~(MODIFIER_SHIFT | MODIFIER_NUML))
+      return 0;
+    if (!event->string)
+      return 0;
+    for (i = 0; i < n; i += 2) {
+      if (!memcmp (event->string, k + i, 2))
+        break;
     }
+    if (i >= n)
+      return 0;
+  } else { /* WIDGET_EVENT_CLICK */
+    if (event->button != 1)
+      return 0;
+  }
+  if (wp->callback) {
+    if (!event->pressed)
+      wp->callback (&wp->w, wp->userdata);
+    ret = 1;
   }
   return ret;
 }
@@ -512,7 +527,8 @@ static int notify_event (xitk_widget_t *w, const widget_event_t *event) {
       }
       break;
     case WIDGET_EVENT_CLICK:
-      return _label_click (wp, event->button, !event->pressed, event->x, event->y);
+    case WIDGET_EVENT_KEY:
+      return _label_input (wp, event);
     case WIDGET_EVENT_CHANGE_SKIN:
       _label_new_skin (wp, event->skonfig);
       break;
@@ -535,7 +551,7 @@ static int notify_event (xitk_widget_t *w, const widget_event_t *event) {
  */
 static xitk_widget_t *_label_create (_label_private_t *wp, const xitk_label_widget_t *l) {
   wp->callback = l->callback;
-  wp->userdata = l->userdata;
+  wp->userdata = l->nw.userdata;
 
   wp->anim_running = 0;
 
@@ -559,23 +575,21 @@ static xitk_widget_t *_label_create (_label_private_t *wp, const xitk_label_widg
 /*
  *
  */
-xitk_widget_t *xitk_label_create (xitk_widget_list_t *wl, xitk_skin_config_t *skonfig,
-  const xitk_label_widget_t *l) {
+xitk_widget_t *xitk_label_create (const xitk_label_widget_t *l, xitk_skin_config_t *skonfig) {
   const xitk_skin_element_info_t *info;
   _label_private_t *wp;
 
-  ABORT_IF_NULL (wl);
-  XITK_CHECK_CONSTITENCY (l);
+  _xitk_new_widget_check (&l->nw);
 
-  wp = (_label_private_t *)xitk_widget_new (wl, sizeof (*wp));
+  wp = (_label_private_t *)xitk_widget_new (l->nw.wl, sizeof (*wp));
   if (!wp)
     return NULL;
   strlcpy (wp->skin_element_name,
-    l->skin_element_name && l->skin_element_name[0] ? l->skin_element_name : "-",
+    l->nw.skin_element_name && l->nw.skin_element_name[0] ? l->nw.skin_element_name : "-",
     sizeof (wp->skin_element_name));
   xitk_short_string_init (&wp->fontname);
 
-  info = xitk_skin_get_info (skonfig, l->skin_element_name);
+  info = xitk_skin_get_info (skonfig, l->nw.skin_element_name);
   if (!info || !info->label_pixmap_font_img) {
     /* not used by this skin || wrong pixmmap name in skin? */
     wp->w.x           = 0;
@@ -624,14 +638,13 @@ xitk_widget_t *xitk_label_create (xitk_widget_list_t *wl, xitk_skin_config_t *sk
 /*
  *
  */
-xitk_widget_t *xitk_noskin_label_create (xitk_widget_list_t *wl,
-  const xitk_label_widget_t *l, int x, int y, int width, int height, const char *fontname) {
+xitk_widget_t *xitk_noskin_label_create (const xitk_label_widget_t *l,
+  int x, int y, int width, int height, const char *fontname) {
   _label_private_t *wp;
 
-  ABORT_IF_NULL (wl);
-  XITK_CHECK_CONSTITENCY (l);
+  _xitk_new_widget_check (&l->nw);
 
-  wp = (_label_private_t *)xitk_widget_new (wl, sizeof (*wp));
+  wp = (_label_private_t *)xitk_widget_new (l->nw.wl, sizeof (*wp));
   if (!wp)
     return NULL;
 
@@ -651,13 +664,13 @@ xitk_widget_t *xitk_noskin_label_create (xitk_widget_list_t *wl,
   xitk_short_string_init (&wp->fontname);
   xitk_short_string_set (&wp->fontname, fontname);
 
-  if (l->skin_element_name && !strcmp (l->skin_element_name, "XITK_NOSKIN_INNER")) {
-    if (xitk_shared_image (wl, "xitk_label_i", width, height, &wp->font) == 1) {
+  if (l->nw.skin_element_name && !strcmp (l->nw.skin_element_name, "XITK_NOSKIN_INNER")) {
+    if (xitk_shared_image (wp->w.wl, "xitk_label_i", width, height, &wp->font) == 1) {
       xitk_image_draw_flat (wp->font, width, height);
       xitk_image_draw_rectangular_box (wp->font, 0, 0, width, height, DRAW_INNER);
     }
   } else {
-    if (xitk_shared_image (wl, "xitk_label_f", width, height, &wp->font) == 1)
+    if (xitk_shared_image (wp->w.wl, "xitk_label_f", width, height, &wp->font) == 1)
       xitk_image_draw_flat (wp->font, width, height);
   }
 
