@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2021 the xine project
+ * Copyright (C) 2000-2022 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -914,6 +914,7 @@ static __gfx_t *_xitk_gfx_new (__xitk_t *xitk) {
 
   xitk_dnode_init (&fx->wl.node);
   xitk_dlist_init (&fx->wl.list);
+  memcpy (&fx->wl.magic, XITK_WL_MAGIC, 4);
   fx->wl.xitk               = &xitk->x;
   fx->wl.xwin               = NULL;
   fx->wl.shared_images      = NULL;
@@ -1277,6 +1278,13 @@ static void xitk_handle_event (__xitk_t *xitk, xitk_be_event_t *event) {
 
   handled = 0;
   switch (event->type) {
+    static const uint8_t keyflags[XITK_KEY_LASTCODE + 1] = {
+      [XITK_KEY_ESCAPE] = 1,
+      [XITK_KEY_RETURN] = 1,
+      [XITK_KEY_TAB] = 2,
+      [XITK_KEY_LEFT_TAB] = 4,
+      [XITK_KEY_MENU] = 8
+    };
 
     case XITK_EV_CLIP_READY:
       if (xwin && xitk->clipboard.widget_in) {
@@ -1300,36 +1308,40 @@ static void xitk_handle_event (__xitk_t *xitk, xitk_be_event_t *event) {
 
     case XITK_EV_KEY_UP:
       gettimeofday (&xitk->keypress, 0);
-      if (fx && fx->wl.widget_focused)
-        handled = xitk_widget_key_event (fx->wl.widget_focused, event->utf8, event->qual, 1);
+      if (fx) {
+        xitk_widget_t *w = fx->wl.widget_focused;
+
+        if (w) {
+          handled = xitk_widget_key_event (w, event->utf8, event->qual, 1);
+          /* this may have changed focus or deleted w. */
+          if (w == fx->wl.widget_focused) {
+            const uint8_t *kstr = (const uint8_t *)event->utf8;
+            uint8_t flags = (kstr[0] == XITK_CTRL_KEY_PREFIX) ? keyflags[kstr[1]] : 0;
+            /* convenience HACK: leaving inputtext with ESCAPE or ENTER shall
+             * set focus to next widget. using XITK_EV_KEY_UP to avoid various glitches. */
+            if ((flags & 1) && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
+              handled = 1;
+              xitk_set_focus_to_next_widget (&fx->wl, event->qual & MODIFIER_SHIFT, event->qual);
+            }
+          }
+        }
+      }
       break;
 
     case XITK_EV_KEY_DOWN:
       if (fx) {
-        static const uint8_t t[XITK_KEY_LASTCODE + 1] = {
-          [XITK_KEY_ESCAPE] = 32,
-          [XITK_KEY_TAB] = 1,
-          [XITK_KEY_KP_TAB] = 1,
-          [XITK_KEY_ISO_LEFT_TAB] = 1,
-          [XITK_KEY_RETURN] = 2,
-          [XITK_KEY_NUMPAD_ENTER] = 2,
-          [XITK_KEY_ISO_ENTER] = 2,
-          [XITK_KEY_UP] = 8,
-          [XITK_KEY_DOWN] = 16,
-          [XITK_KEY_PREV] = 8,
-          [XITK_KEY_NEXT] = 16,
-          [XITK_KEY_LASTCODE] = 4
-        };
-        uint8_t       *kbuf = (uint8_t *)event->utf8;
+        const uint8_t *kstr = (const uint8_t *)event->utf8;
         int            modifier = event->qual;
+        uint8_t        flags;
         xitk_widget_t *w;
 
         xitk_tips_hide_tips (xitk->x.tips);
 
         w = fx->wl.widget_focused;
 
-        /* hint possible menu location */
-        if ((event->utf8[0] == XITK_CTRL_KEY_PREFIX) && (event->utf8[1] == XITK_KEY_MENU) && fx->wl.xwin->bewin) {
+        flags = (kstr[0] == XITK_CTRL_KEY_PREFIX) ? keyflags[kstr[1]] : 0;
+        if ((flags & 8) && fx->wl.xwin->bewin) {
+          /* hint possible menu location */
           xitk_tagitem_t tags[] = {{XITK_TAG_X, 0}, {XITK_TAG_Y, 0}, {XITK_TAG_END, 0}};
           fx->wl.xwin->bewin->get_props (fx->wl.xwin->bewin, tags);
           event->w = tags[0].value;
@@ -1347,23 +1359,19 @@ static void xitk_handle_event (__xitk_t *xitk, xitk_be_event_t *event) {
         /* this may have changed focus or deleted w. */
         w = fx->wl.widget_focused;
 
-        if (!handled) {
-          if (kbuf[0] == ' ')
-            kbuf[0] = XITK_CTRL_KEY_PREFIX, kbuf[1] = XITK_KEY_LASTCODE;
-          if (kbuf[0] == XITK_CTRL_KEY_PREFIX) {
-            if ((t[kbuf[1]] == 1) || (w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT) && (t[kbuf[1]] & 34))) {
-              handled = 1;
-              xitk_set_focus_to_next_widget (&fx->wl, (modifier & MODIFIER_SHIFT), modifier);
-            }
-          }
+        if (!handled && (flags & (2 | 4))) {
+          handled = 1;
+          /* TJ. I have seen keyboards with separate left tab key.
+           * My keyboards have not. However, when I press shift + tab,
+           * I get MODIFIER_SHIFT + XITK_KEY_LEFT_TAB. */
+          if (flags & 4) /* XITK_KEY_LEFT_TAB */
+            modifier |= MODIFIER_SHIFT;
+          xitk_set_focus_to_next_widget (&fx->wl, (modifier & MODIFIER_SHIFT), modifier);
+          w = fx->wl.widget_focused;
         }
 
-        if (!handled) {
-          if (xitk->menu) {
-            if (!(fx->wl.widget_focused && (fx->wl.widget_focused->type & WIDGET_GROUP_MENU)))
-              xitk_set_current_menu (&xitk->x, NULL);
-          }
-        }
+        if (!handled && xitk->menu && !(w && (w->type & WIDGET_GROUP_MENU)))
+          xitk_set_current_menu (&xitk->x, NULL);
       }
       break;
 
@@ -1917,7 +1925,7 @@ char *xitk_filter_filename(const char *name) {
   if (!strncasecmp (name, "file:", 5)) {
     const uint8_t *p = (const uint8_t *)name + 5;
     uint8_t *ret, *q;
-    size_t l = strlen ((const char *)p);
+    size_t l = xitk_find_byte ((const char *)p, 0);
     ret = malloc (l + 2);
     if (!ret)
       return NULL;
@@ -1983,6 +1991,7 @@ long int xitk_get_last_keypressed_time (xitk_t *xitk) {
 char *xitk_cfg_load (const char *filename, size_t *filesize) {
   char *buf;
   size_t fsize;
+  long   flen;
   FILE *f;
 
   if (!filename) {
@@ -2005,8 +2014,8 @@ char *xitk_cfg_load (const char *filename, size_t *filesize) {
     errno = e;
     return NULL;
   }
-  fsize = ftell (f);
-  if (fseek (f, 0, SEEK_SET)) {
+  fsize = flen = ftell (f);
+  if (flen < 0 || fseek (f, 0, SEEK_SET)) {
     int e = errno;
     fclose (f);
     errno = e;
@@ -2182,6 +2191,7 @@ xitk_cfg_parse_t *xitk_cfg_parse (char *contents, int flags) {
 
   box = tree;
   box->level = 0;
+  box->klen = 0;
   box->next = box->prev = box->parent = box->first_child = box->last_child = 0;
   box->key = box->value = -1;
   used = 1;
@@ -2232,9 +2242,10 @@ xitk_cfg_parse_t *xitk_cfg_parse (char *contents, int flags) {
         } else {
           uint8_t *key = p;
           item->key = p - start;
-          item = NULL;
           while (!((z = tab_cfg_parse[*p]) & 0xfd))
             p++;
+          item->klen = p - key;
+          item = NULL;
           if (!(flags & XITK_CFG_PARSE_CASE))
             _xitk_lower (key, p - key);
           *e = 0;
@@ -2316,6 +2327,7 @@ xitk_cfg_parse_t *xitk_cfg_parse (char *contents, int flags) {
         }
         item = tree + used;
         item->level = box->level + 1;
+        item->klen = 0;
         item->parent = box - tree;
         item->next = 0;
         item->prev = box->last_child;
@@ -2333,6 +2345,8 @@ xitk_cfg_parse_t *xitk_cfg_parse (char *contents, int flags) {
   }
   *e = 0;
 
+  tree->key = used;
+
   if (flags & XITK_CFG_PARSE_DEBUG) {
     int i;
 
